#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

import os
import sys
import setenv

kernel_root=os.getenv("KERNEL_ROOT_DIR")
sys.path[:0]=[kernel_root+"/bin/salome"]

argv = sys.argv[1:]
sys.argv = argv[1:]

if argv[0] == "-exec":
  #keep options only for execution
  del argv[0]
  sys.argv = []

setenv.main(True);

if sys.platform == "win32":
    if len(argv) >= 2:
        # Fix a problem of spaces in argv[1]
        argv[1] = '"%s"'%(argv[1])
        pass
    pass

os.execvp(argv[0],argv)



#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

import sys,os

def getNSparams(info=""):
    """
    check environment for omniORB configuration file.
    parse the file to find the line defining naming service  host and port,
    set environment variables NSPORT and NSHOST,
    get host and port,
    if   info==host print host
    elif info==port print host
    else    print 2 strings on stdout on one line: host port
    """
    from salome_utils import getORBcfgInfo
    my_version, my_host, my_port = getORBcfgInfo()
    if info=='host':
        # keep print, stdout used in shell
        print my_host
        os.environ['NSHOST']=my_host
        return my_host
        pass
    elif info=='port':
        # keep print, stdout used in shell
        print my_port
        os.environ['NSPORT']=my_port
        return my_port
        pass
    else:
        # keep print, stdout used in shell
        print  my_host, my_port
        return my_host, my_port
    pass

# ------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) >1:        
        if sys.argv[1]=='host':
            getNSparams('host')
            pass
        elif sys.argv[1]=='port':
            getNSparams('port')
            pass
        else:
            getNSparams()
            pass
        pass
    else:
        getNSparams()
        pass
    pass



#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

## @package orbmodule
# \brief Module that provides a client for %SALOME
#

import sys,os,time
import string
from nameserver import *
from omniORB import CORBA
from launchConfigureParser import verbose

# Import the stubs for the Naming service
import CosNaming
#from runNS import *

# -----------------------------------------------------------------------------

class client:
   """Client for SALOME"""

   def __init__(self,args=None):
      #set GIOP message size for bug 10560: impossible to get field values in TUI mode
      sys.argv.extend(["-ORBgiopMaxMsgSize", "104857600"]) ## = 100 * 1024 * 1024
      # Initialise the ORB
      self.orb=CORBA.ORB_init(sys.argv, CORBA.ORB_ID)
      # Initialise the Naming Service
      self.initNS(args or {})

   # --------------------------------------------------------------------------

   def initNS(self,args):
      # Obtain a reference to the root naming context
      obj         = self.orb.resolve_initial_references("NameService")
      try:
          self.rootContext = obj._narrow(CosNaming.NamingContext)
          return
      except (CORBA.TRANSIENT,CORBA.OBJECT_NOT_EXIST,CORBA.COMM_FAILURE):
          self.rootContext = None
          if verbose(): print "Launch Naming Service++",
          
      # On lance le Naming Server (doit etre dans le PATH)
      test = True
      if args['wake_up_session']:
         test = False
         pass
      if test:
         NamingServer(args).run()
         pass
      print "Searching Naming Service ",
      ncount=0
      delta=0.1
      while(ncount < 100):
          ncount += 1
          try:
              obj = self.orb.resolve_initial_references("NameService")
              self.rootContext = obj._narrow(CosNaming.NamingContext)
              break
          except (CORBA.TRANSIENT,CORBA.OBJECT_NOT_EXIST,CORBA.COMM_FAILURE):
              self.rootContext = None
              sys.stdout.write('+')
              sys.stdout.flush()
              time.sleep(delta)

      if self.rootContext is None:
          print "Failed to narrow the root naming context"
          sys.exit(1)
      print " found in %s seconds " % ((ncount-1)*delta)

   # --------------------------------------------------------------------------

   def showNScontext(self,context,dec=''):
      bl,bi=context.list(0)
      if bi is not None:
         ok,b=bi.next_one()
         while(ok):
            for s in b.binding_name :
               print "%s%s.%s" %(dec,s.id,s.kind)
               if s.kind == "dir":
                  obj=context.resolve([s])
                  scontext = obj._narrow(CosNaming.NamingContext)
                  self.showNScontext(scontext,dec=dec+'  ')
            ok,b=bi.next_one()

   # --------------------------------------------------------------------------

   def showNS(self):
      """ Show the content of SALOME naming service """
      self.showNScontext(self.rootContext)

   # --------------------------------------------------------------------------

   def Resolve(self, Path):
      resolve_path=string.split(Path,'/')
      if resolve_path[0] == '': del resolve_path[0]
      dir_path=resolve_path[:-1]
      context_name=[]
      for e in dir_path:
         context_name.append(CosNaming.NameComponent(e,"dir"))
      context_name.append(CosNaming.NameComponent(resolve_path[-1],"object"))

      try:
          obj = self.rootContext.resolve(context_name)
      except CosNaming.NamingContext.NotFound, ex:
          obj = None
      except CosNaming.NamingContext.InvalidName, ex:
          obj = None
      except CosNaming.NamingContext.CannotProceed, ex:
          obj = None
      except (CORBA.TRANSIENT,CORBA.OBJECT_NOT_EXIST,CORBA.COMM_FAILURE):
          obj = None
      return obj

   # --------------------------------------------------------------------------

   def waitNS(self,name,typobj=None,maxcount=240):
      count=0
      delta=0.5
      print "Searching %s in Naming Service " % name,
      while(1):
          count += 1
          if count > maxcount : raise RuntimeError, "Impossible de trouver %s" % name
          obj=self.Resolve(name)
          if obj : 
              print " found in %s seconds " % ((count-1)*delta)
              break
          else:
              sys.stdout.write('+')
              sys.stdout.flush()
              time.sleep(delta)
 
      if typobj is None:return obj

      nobj = obj._narrow(typobj)
      if nobj is None:
            print "%s exists but is not a %s" % (name,typobj)
      return nobj

   if sys.platform != "win32":
    def waitNSPID(self, theName, thePID, theTypObj = None):
      aCount = 0
      aDelta = 0.5
      anObj = None
      print "Searching %s in Naming Service " % theName,
      while(1):
         try:
           os.kill(thePID,0)
         except:
           raise RuntimeError, "Process %d for %s not found" % (thePID,theName)
         aCount += 1
         anObj = self.Resolve(theName)
         if anObj: 
            print " found in %s seconds " % ((aCount-1)*aDelta)
            break
         else:
            sys.stdout.write('+')
            sys.stdout.flush()
            time.sleep(aDelta)
            pass
         pass
      
      if theTypObj is None:
         return anObj

      anObject = anObj._narrow(theTypObj)
      if anObject is None:
         print "%s exists but is not a %s" % (theName,theTypObj)
      return anObject


   # --------------------------------------------------------------------------

   def ResolveLogger(self, name):
      context_name=[]
      context_name.append(CosNaming.NameComponent(name,""))

      try:
          obj = self.rootContext.resolve(context_name)
      except CosNaming.NamingContext.NotFound, ex:
          obj = None
      except CosNaming.NamingContext.InvalidName, ex:
          obj = None
      except CosNaming.NamingContext.CannotProceed, ex:
          obj = None
      except (CORBA.TRANSIENT,CORBA.OBJECT_NOT_EXIST,CORBA.COMM_FAILURE):
          obj = None
      return obj
   
   # --------------------------------------------------------------------------

   def waitLogger(self,name,typobj=None,maxcount=40):
      count=0
      delta=0.5
      print "Searching %s in Naming Service " % name,
      while(1):
          count += 1
          if count > maxcount : raise RuntimeError, "Impossible de trouver %s" % name
          obj=self.ResolveLogger(name)
          if obj : 
              print " found in %s seconds " % ((count-1)*delta)
              break
          else:
              sys.stdout.write('+')
              sys.stdout.flush()
              time.sleep(delta)
 
      if typobj is None:return obj

      nobj = obj._narrow(typobj)
      if nobj is None:
            print "%s exists but is not a %s" % (name,typobj)
      return nobj




# -*- coding: iso-8859-1 -*-
# Copyright (C) 2010-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
__author__="gboulant"
__date__ ="$1 avr. 2010 09:45:21$"

import sys

def run(modulePath, functionName):
    """
    This function should be used for very basic unit tests and only for a
    rapid development. A better way should be the pyunit framework.
    The function functionName is supposed here to return a boolean value
    indicating if the test is OK (True) or NOT OK (False)
    """
    moduleName = modulePath.replace('/','.') 
    __import__ (moduleName)
    module=sys.modules[moduleName]
    func = getattr(module,functionName)
    tabsize = 70-len(moduleName)-len(functionName)
    print "[TEST] %s.%s %s test in progress" % (moduleName, functionName,"."*tabsize) 
    ok = func()
    if ( ok ):
        print "[TEST] %s.%s %s OK" % (moduleName, functionName,"."*tabsize)
    else:
        print "[TEST] %s.%s %s NOT OK" % (moduleName, functionName,"."*tabsize)

def tryfunction(function,*argv):
    """
    This function is for debug only. It executes the specified function with the
    specified arguments in a try/except bloc so that to display the exception in
    the case where an exception is raised (usefull to debug server side of a CORBA
    process).
    """
    print "[TEST] trying the function %s" % function
    try:
        return function(*argv)
    except Exception, e:
        print e
        raise e


#
# ==============================================================================
# Simple use cases
# ==============================================================================
#
def TEST_myTestIsOk():
    return True

def TEST_myTestIsNotOk():
    return False

if __name__ == "__main__":
    run("salome/kernel/unittester","TEST_myTestIsOk")
    run("salome/kernel/unittester","TEST_myTestIsNotOk")



#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

## \file killSalomeWithPort.py
#  Stop all %SALOME servers from given sessions by killing them
#
#  The sessions are indicated by their ports on the command line as in :
#  \code
#  killSalomeWithPort.py 2811 2815
#  \endcode
#

import os, sys, pickle, signal, commands,glob
from salome_utils import verbose
import Utils_Identity
import salome_utils

def getPiDict(port,appname='salome',full=True,hidden=True,hostname=None):
    """
    Get file with list of SALOME processes.
    This file is located in the user's home directory
    and named .<user>_<host>_<port>_SALOME_pidict
    where
    <user> is user name
    <host> is host name
    <port> is port number

    Parameters:
    - port    : port number
    - appname : application name (default is 'SALOME')
    - full    : if True, full path to the file is returned, otherwise only file name is returned
    - hidden  : if True, file name is prefixed with . (dot) symbol; this internal parameter is used
    to support compatibility with older versions of SALOME
    """
    from salome_utils import generateFileName, getTmpDir
    dir = ""
    if not hostname:
        hostname = os.getenv("NSHOST")
        if hostname: hostname = hostname.split(".")[0]
        pass
    if full:
        # full path to the pidict file is requested
        if hidden:
            # new-style dot-prefixed pidict files
            # are in the system-dependant temporary diretory
            dir = getTmpDir()
        else:
            # old-style non-dot-prefixed pidict files
            # are in the user's home directory
            dir = os.getenv("HOME")
            pass
        pass
    return generateFileName(dir,
                            suffix="pidict",
                            hidden=hidden,
                            with_username=True,
                            with_hostname=hostname or True,
                            with_port=port,
                            with_app=appname.upper())

def appliCleanOmniOrbConfig(port):
    """
    Remove omniorb config files related to the port in SALOME application:
    - ${HOME}/${APPLI}/USERS/.omniORB_${USER}_${HOSTNAME}_${NSPORT}.cfg
    - ${HOME}/${APPLI}/USERS/.omniORB_${USER}_last.cfg
    the last is removed only if the link points to the first file.
    """
    from salome_utils import generateFileName, getUserName
    home  = os.getenv("HOME")
    appli = os.getenv("APPLI")
    if appli is None:
        #Run outside application context
        pass
    else:
        dir = os.path.join(home, appli,"USERS")
        omniorb_config      = generateFileName(dir, prefix="omniORB",
                                               extension="cfg",
                                               hidden=True,
                                               with_username=True,
                                               with_hostname=True,
                                               with_port=port)
        last_running_config = generateFileName(dir, prefix="omniORB",
                                               with_username=True,
                                               suffix="last",
                                               extension="cfg",
                                               hidden=True)
        if os.access(last_running_config,os.F_OK):
            pointedPath = os.readlink(last_running_config)
            if pointedPath[0] != '/':
                pointedPath=os.path.join(os.path.dirname(last_running_config), pointedPath)
            if pointedPath == omniorb_config:
                os.unlink(last_running_config)
                pass
            pass
        if os.access(omniorb_config,os.F_OK):
            os.remove(omniorb_config)
            pass

        if os.path.lexists(last_running_config):return 

        #try to relink last.cfg to an existing config file if any
        files = glob.glob(os.path.join(os.environ["HOME"],Utils_Identity.getapplipath(),
                                       "USERS",".omniORB_"+getUserName()+"_*.cfg"))
        current_config=None
        current=0
        for f in files:
          stat=os.stat(f)
          if stat.st_atime > current:
            current=stat.st_atime
            current_config=f
        if current_config:
          os.symlink(os.path.normpath(current_config), last_running_config)

        pass
    pass

########## kills all salome processes with the given port ##########

def shutdownMyPort(port):
    """
    Shutdown SALOME session running on the specified port.
    Parameters:
    - port - port number
    """
    if not port: return
    
    from salome_utils import generateFileName

    # set OMNIORB_CONFIG variable to the proper file
    home  = os.getenv("HOME")
    appli = os.getenv("APPLI")
    kwargs = {}
    if appli is not None: 
        home = os.path.join(home, appli,"USERS")
        kwargs["with_username"]=True
        pass
    omniorb_config = generateFileName(home, prefix="omniORB",
                                      extension="cfg",
                                      hidden=True,
                                      with_hostname=True,
                                      with_port=port,
                                      **kwargs)
    os.environ['OMNIORB_CONFIG'] = omniorb_config

    # give the chance to the servers to shutdown properly
    try:
        import time
        import salome_kernel
        orb, lcc, naming_service, cm = salome_kernel.salome_kernel_init()
        # shutdown all
        lcc.shutdownServers()
        # give some time to shutdown to complete
        time.sleep(1)
        # shutdown omniNames and notifd
        salome_kernel.LifeCycleCORBA.killOmniNames()
    except:
        pass
    pass
    
def killMyPort(port):
    """
    Kill SALOME session running on the specified port.
    Parameters:
    - port - port number
    """
    from salome_utils import getShortHostName, getHostName
    
    # try to shutdown session nomally
    import threading, time
    threading.Thread(target=shutdownMyPort, args=(port,)).start()
    time.sleep(3) # wait a little, then kill processes (should be done if shutdown procedure hangs up)
    
    # new-style dot-prefixed pidict file
    filedict = getPiDict(port, hidden=True)
    # provide compatibility with old-style pidict file (not dot-prefixed)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=False)
    # provide compatibility with old-style pidict file (short hostname)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=True,  hostname=getShortHostName())
    # provide compatibility with old-style pidict file (not dot-prefixed, short hostname)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=False, hostname=getShortHostName())
    # provide compatibility with old-style pidict file (long hostname)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=True,  hostname=getHostName())
    # provide compatibility with old-style pidict file (not dot-prefixed, long hostname)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=False, hostname=getHostName())
    #
    try:
        fpid = open(filedict, 'r')
        #
        from salome_utils import generateFileName
        if sys.platform == "win32":
            username = os.getenv( "USERNAME" )
        else:
            username = os.getenv('USER')
        path = os.path.join('/tmp/logs', username)
        fpidomniNames = generateFileName(path,
                                         prefix="",
                                         suffix="Pid_omniNames",
                                         extension="log",
                                         with_port=port)
        if not sys.platform == 'win32':        
            cmd = 'pid=`ps -eo pid,command | egrep "[0-9] omniNames -start %s"` ; echo $pid > %s' % ( str(port), fpidomniNames )
            a = os.system(cmd)
            pass
        try:
            fpidomniNamesFile = open(fpidomniNames)
            lines = fpidomniNamesFile.readlines()
            fpidomniNamesFile.close()
            os.remove(fpidomniNames)
            for l in lines:
                try:
                    pidfield = l.split()[0] # pid should be at the first position
                    if sys.platform == "win32":
                        import win32pm
                        if verbose(): print 'stop process '+pidfield+' : omniNames'
                        win32pm.killpid(int(pidfield),0)
                    else:
                        if verbose(): print 'stop process '+pidfield+' : omniNames'
                        os.kill(int(pidfield),signal.SIGKILL)
                        pass
                    pass
                except:
                    pass
                pass
            pass
        except:
            pass
        #
        try:
            process_ids=pickle.load(fpid)
            fpid.close()
            for process_id in process_ids:
                for pid, cmd in process_id.items():
                    if verbose(): print "stop process %s : %s"% (pid, cmd[0])
                    try:
                        if sys.platform == "win32":
                            import win32pm
                            win32pm.killpid(int(pid),0)                            
                        else:
                            os.kill(int(pid),signal.SIGKILL)
                            pass
                        pass
                    except:
                        if verbose(): print "  ------------------ process %s : %s not found"% (pid, cmd[0])
                        pass
                    pass # for pid, cmd ...
                pass # for process_id ...
            pass # try...
        except:
            pass
        #
        os.remove(filedict)
        cmd='ps -eo pid,command | egrep "[0-9] omniNames -start '+str(port)+'" | sed -e "s%[^0-9]*\([0-9]*\) .*%\\1%g"'
        pid = commands.getoutput(cmd)
        a = ""
        while pid and len(a.split()) < 2:
            a = commands.getoutput("kill -9 " + pid)
            pid = commands.getoutput(cmd)
            #print pid
            pass
        pass
    except:
        print "Cannot find or open SALOME PIDs file for port", port
        pass
    #
    appliCleanOmniOrbConfig(port)
    pass
            
def killNotifdAndClean(port):
    """
    Kill notifd daemon and clean application running on the specified port.
    Parameters:
    - port - port number
    """
    try:
      filedict=getPiDict(port)
      f=open(filedict, 'r')
      pids=pickle.load(f)
      for d in pids:
        for pid,process in d.items():
          if 'notifd' in process:
            cmd='kill -9 %d'% pid
            os.system(cmd)
      os.remove(filedict)
    except:
      #import traceback
      #traceback.print_exc()
      pass

    appliCleanOmniOrbConfig(port)

def killMyPortSpy(pid, port):
    dt = 1.0
    while 1:
        if sys.platform == "win32":
            from win32pm import killpid
            if killpid(int(pid), 0) != 0:
                return
        else:
            from os import kill
            try:
                kill(int(pid), 0)
            except OSError, e:
                if e.errno != 3:
                    return
                break
            pass
        from time import sleep
        sleep(dt)
        pass
    filedict = getPiDict(port, hidden=True)
    if not os.path.exists(filedict):
        return
    try:
        import omniORB
        orb = omniORB.CORBA.ORB_init(sys.argv, omniORB.CORBA.ORB_ID)
        import SALOME_NamingServicePy
        ns = SALOME_NamingServicePy.SALOME_NamingServicePy_i(orb)
        import SALOME
        session = ns.Resolve("/Kernel/Session")
        assert session
    except:
        return
    try:
        status = session.GetStatSession()
    except:
        # -- session is in naming service but has crash
        status = None
        pass
    if status:
        if not status.activeGUI:
            return
        pass
    killMyPort(port)
    return

if __name__ == "__main__":
    if sys.argv[1] == "--spy":
        pid = sys.argv[2]
        port = sys.argv[3]
        killMyPortSpy(pid, port)
        sys.exit(0)
        pass
    for port in sys.argv[1:]:
        killMyPort(port)
        pass
    pass



# -*- coding: iso-8859-1 -*-
# Copyright (C) 2011-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

import sys
from salome.kernel import termcolor
def printfile(filename):
    stream = open(filename,'r')
    lines = stream.readlines()
    stream.close()
    for line in lines:
	if not termcolor.canDisplayColor(sys.stdout):
	    msg = line.split('\n')[0]
	else:
            msg = termcolor.makeColoredMessage(line.split('\n')[0], termcolor.BLUE)
        print msg

import os
import unittest
from unittest import TestCase
from uiexception import DevelException

class PyUnitTester():
    '''
    This class is a simple wrapper of the pyunit framework.
    '''
    def __init__(self):
        self.__listTestSuite=[]
        self.setLogFilename("testlog.txt")

    def addTestCase(self,testCaseClass):
        # We test some attributes to validate that the argument
        # corresponds to a unittest.TestCase class (WARN: it's not an
        # instance but a class keyword)
        if "assertTrue" not in dir(testCaseClass):
            raise DevelException("Not a pyunit test case")

        suite = unittest.TestLoader().loadTestsFromTestCase(testCaseClass)
        self.__listTestSuite.append(suite)

    def setLogFilename(self, filename):
        self.__logfilename = filename

    def run(self):
        # We first open the output stream
        if os.path.exists(self.__logfilename):
            os.remove(self.__logfilename)
        logfile = open(self.__logfilename,'w')
        
        # Then, define and execute the runner to play the test suites
        runner = unittest.TextTestRunner(stream=logfile,verbosity=2)
        for suite in self.__listTestSuite:
            runner.run(suite)

        # Finally close the ouput stream and print the report
        logfile.close()
        self.printreport()

    def printreport(self):
        printfile(self.__logfilename)

def execAndConvertExceptionToBoolean(function):
    """
    This can be used to just execute a test function that is
    considered as OK simply if no exception is raised (can be test by
    an assertTrue).
    """
    try:
        result = function()
        if result is None:
            return True
        return result
    except Exception, e:
        print e
        return False

# Simple helper function for most cases where there is only one
# TestCase classe.
def run(testCaseClass):
    """
    Run the test suite provided by the specified TestCase class.
    """
    tester = PyUnitTester()
    tester.addTestCase(testCaseClass)
    tester.run()

#
# ==============================================================================
# Simple use cases
# ==============================================================================
#
class MyTestCase(unittest.TestCase):
    def test_myTestOk_01(self):
        r=True
        print "myTestOk_01: should be OK"
        self.assertTrue(r)

    def test_myTestOk_02(self):
        r=True
        print "myTestOk_02: should be OK"
        self.assertTrue(r)

    def test_myTestNotOk(self):
        r=False
        print "myTestNotOk: should be NOT OK"
        self.assertTrue(r)

def functionRaisingAnException():
    raise Exception("An error occurs")

def functionReturningFalse():
    return False

def functionReturningTrue():
    return True

def functionReturningNothing():
    print "functionReturningNothing"

class MyTestCase2(unittest.TestCase):
    def test_myTest_01(self):
        r=execAndConvertExceptionToBoolean(functionRaisingAnException)
        print "test 01: this test should be NOT OK"
        self.assertTrue(r)

    def test_myTest_02(self):
        r=execAndConvertExceptionToBoolean(functionReturningFalse)
        print "test 02: this test should be NOT OK"
        self.assertTrue(r)

    def test_myTest_03(self):
        r=execAndConvertExceptionToBoolean(functionReturningTrue)
        print "test 03: this test should be OK"
        self.assertTrue(r)

    def test_myTest_04(self):
        r=execAndConvertExceptionToBoolean(functionReturningNothing)
        print "test 04: this test should be OK"
        self.assertTrue(r)

    def test_myTest_05(self):
        r=True
        print "test 05: this test should be OK"
        self.assertTrue(r)


def TEST_basic_usecase():
    tester = PyUnitTester()
    # Note that the class keywords must be passed
    tester.addTestCase(MyTestCase)
    tester.addTestCase(MyTestCase2)
    tester.run()
    
if __name__ == "__main__":
    TEST_basic_usecase()



#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

import os, sys, pickle, string, signal
from launchConfigureParser import verbose

########## adds to the kill list of SALOME one more process ##########

def findFileDict():
    """
    Detect current SALOME session's port number.
    Returns port number.
    """
    from salome_utils import getPortNumber
    port = getPortNumber()
    if verbose(): print "myport = ", port
    return port
    
def addToKillList(command_pid, command, port=None):
    """
    Add the process to the SALOME processes dictionary file.
    Parameters:
    - command_pid : command PID
    - command     : command (string or list of strings)
    - [port]      : SALOME port number; if this parameter is None (default),
    it is detected automatically
    """
    # retrieve current processes dictionary
    from killSalomeWithPort import getPiDict
    if port is None: port=findFileDict()
    filedict=getPiDict(port)
    try:
        fpid=open(filedict, 'r')
        process_ids=pickle.load(fpid)
        fpid.close()
    except:
        process_ids=[]
        pass
    # check if PID is already in dictionary
    already_in=False
    for process_id in process_ids:
        for pid, cmd in process_id.items():
            if int(pid) == int(command_pid):
                already_in=True
                break
            pass
        if already_in: break
        pass
    # add process to the dictionary
    if not already_in:
        import types
        if type(command) == types.ListType: command=" ".join([str(c) for c in command])
        command=command.split()[0]
        try:
            if verbose(): print "addToKillList: %s : %s" % ( str(command_pid), command )
            process_ids.append({int(command_pid): [command]})
            dir = os.path.dirname(filedict)
            if not os.path.exists(dir): os.makedirs(dir, 0777)
            fpid = open(filedict,'w')
            pickle.dump(process_ids, fpid)
            fpid.close()
        except:
            if verbose(): print "addToKillList: can not add command %s : %s to the kill list" % ( str(command_pid), command )
            pass
        pass
    pass

def killList(port=None):
    """
    Kill all the processes listed in the SALOME processes dictionary file.
    - [port]      : SALOME port number; if this parameter is None (default),
    it is detected automatically
    """
    # retrieve processes dictionary
    from killSalomeWithPort import getPiDict
    if port is None: port=findFileDict()
    # new-style dot-prefixed pidict file
    filedict=getPiDict(port, hidden=True)
    # provide compatibility with old-style pidict file (not dot-prefixed)
    if not os.path.exists(filedict): filedict = getPiDict(port, hidden=False)
    try:
        fpid=open(filedict, 'r')
        process_ids=pickle.load(fpid)
        fpid.close()
    except:
        process_ids=[]
        pass
    # kill processes
    for process_id in process_ids:
        #print process_id
        for pid, cmd in process_id.items():
            print "stop process %s : %s"% (pid, cmd[0])
            try:
                os.kill(int(pid),signal.SIGKILL)
            except:
                print "  ------------------ process %s : %s inexistant"% (pid, cmd[0])
                pass
            pass
        pass
    # remove processes dictionary file
    os.remove(filedict)
    pass

if __name__ == "__main__":
    if verbose(): print sys.argv
    addToKillList(sys.argv[1], sys.argv[2])
    pass



#!/usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#log files localization
#
import os, commands, sys, re, string, socket
from Utils_Identity import getShortHostName

if sys.platform == "win32":
  # temporarily using home directory for Namning Service logs
  # to be replaced with TEMP later...
  os.environ["BaseDir"]=os.environ["HOME"]
else:
  os.environ["BaseDir"]="/tmp"

os.environ["Username"]=os.environ["USER"]

# kill OmniNames if exists

#killall -q -9 omniNames

# clear log files

def startOmni():
  try:
    os.mkdir(os.environ["BaseDir"] + "/logs")
    os.chmod(os.environ["BaseDir"] + "/logs", 0777)
  except:
    #print "Can't create " + os.environ["BaseDir"] + "/logs"
    pass
  
  upath = os.environ["BaseDir"] + "/logs/" + os.environ["Username"]

  try:
    os.mkdir(upath)
  except:
    #print "Can't create " + upath
    pass

  #os.system("touch " + upath + "/dummy")
  for fname in os.listdir(upath):
    try:
      if not fname.startswith("."): os.remove(upath + "/" + fname)
    except:
      pass
  #os.system("rm -f " + upath + "/omninames* " + upath + "/dummy " + upath + "/*.log")

  print "Name Service... "
  #hname=os.environ["HOST"] #commands.getoutput("hostname")
  if sys.platform == "win32":
          hname=getShortHostName()
  else:
    hname=socket.gethostname()

  print "hname=",hname
  
  f=open(os.environ["OMNIORB_CONFIG"])
  ss=re.findall("NameService=corbaname::" + hname + ":\d+", f.read())
  print "ss = ", ss
  f.close()
  aPort=re.findall("\d+", ss[0])[0]
  
  #aSedCommand="s/.*NameService=corbaname::" + hname + ":\([[:digit:]]*\)/\1/"
  #print "sed command = ", aSedCommand
  #aPort = commands.getoutput("sed -e\"" + aSedCommand + "\"" + os.environ["OMNIORB_CONFIG"])
  global process_id
  print "port=", aPort
  if sys.platform == "win32":          
          #import win32pm
          #command = ['omniNames -start ' , aPort , ' -logdir ' , '\"' + upath + '\"']
          #os.system("start omniNames -start " + aPort + " -logdir " + "\"" + upath + "\"" )
          command = "start omniNames -start " + aPort + " -logdir " + "\"" + upath + "\""
          #print command
          pid = win32pm.spawnpid( string.join(command, " "), -nc )
          process_id[pid]=command
  else:
    os.system("omniNames -start " + aPort + " -logdir " + upath + " &")

  print "ok"
  print "to list contexts and objects bound into the context with the specified name : showNS "

# In LifeCycleCORBA, FactoryServer is started with rsh on the requested
#    computer if this Container does not exist. Default is localhost.
#    Others Containers are started with start_impl method of FactoryServer Container.
# For using rsh it is necessary to have in the ${HOME} directory a .rhosts file
# Warning : on RedHat the file /etc/hosts contains by default a line like :
# 127.0.0.1               bordolex bordolex.paris1.matra-dtv.fr localhost.localdomain localhost  
#   (bordolex is the station name). omniNames on bordolex will be accessible from other
#   computers only if the computer name is removed on that line like :
#   127.0.0.1               bordolex.paris1.matra-dtv.fr localhost.localdomain localhost

# To start dynamically Containers on several computers you need to
# put in the ${OMNIORB_CONFIG} file a computer name instead of "localhost"
# example : ORBInitRef NameService=corbaname::dm2s0017

# If you need to use several omniNames running on the same computer, you have to :
#1. put in your ${OMNIORB_CONFIG} file a computer name and port number
# example : ORBInitRef NameService=corbaname::dm2s0017:1515
#2. start omninames with this port number in runNS.sh
# example : omniNames -start 1515 -logdir ${BaseDir}/logs/${Username} &



#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  SALOME Container : implementation of container and engine for Kernel
#  File   : SALOME_Container.py
#  Author : Paul RASCLE, EDF
#  Module : SALOME
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/Container/SALOME_Container.py,v 1.5.2.4.14.4.12.1 2012-04-12 14:05:03 vsr Exp $

## @package SALOME_Container
# \brief python implementation of container interface for Kernel
#

import os
import sys
import string
import traceback
import imp
from omniORB import CORBA, PortableServer
import SALOMEDS
import Engines, Engines__POA
from SALOME_NamingServicePy import *
from SALOME_ComponentPy import *
import SALOME_PyNode

from SALOME_utilities import *
from Utils_Identity import getShortHostName
from launchConfigureParser import verbose

#=============================================================================

#define an implementation of the container interface for embedding in Container implemented in C++

class SALOME_Container_i:
    _orb = None
    _poa = None
    _containerName = ""
    _naming_service = None

    #-------------------------------------------------------------------------

    def __init__(self ,containerName, containerIORStr):
        MESSAGE( "SALOME_Container_i::__init__" )
        self._orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)
        self._poa = self._orb.resolve_initial_references("RootPOA")
        self._containerName = containerName
        if verbose(): print "SALOME_Container.SALOME_Container_i : _containerName ",self._containerName
        #self._naming_service = SALOME_NamingServicePy_i(self._orb)
        self._container = self._orb.string_to_object(containerIORStr)

    #-------------------------------------------------------------------------

    def import_component(self, componentName):
        MESSAGE( "SALOME_Container_i::import_component" )
        ret=""
        try:
            if verbose(): print "try import ",componentName
            __import__(componentName)
            if verbose(): print "import ",componentName," successful"
        except ImportError,e:
            #can't import python module componentName
            #try to find it in python path
            try:
              fp, pathname, description = imp.find_module(componentName)
              if fp:fp.close()
              #module file found in path
              ret="Component "+componentName+": Python implementation found but it can't be loaded\n"
              ret=ret+traceback.format_exc(10)
            except ImportError,ee:
              ret="ImplementationNotFound"
            except:
              if verbose():print "error when calling find_module"
              ret="ImplementationNotFound"
        except:
            ret="Component "+componentName+": Python implementation found but it can't be loaded\n"
            ret=ret+traceback.format_exc(10)
            if verbose():
              traceback.print_exc()
              print "import ",componentName," not possible"
        return ret
        
    #-------------------------------------------------------------------------

    def create_component_instance(self, componentName, instanceName, studyId):
        MESSAGE( "SALOME_Container_i::create_component_instance" )
        comp_iors=""
        ret=""
        try:
            component=__import__(componentName)
            factory=getattr(component,componentName)
            comp_i=factory(self._orb,
                           self._poa,
                           self._container,
                           self._containerName,
                           instanceName,
                           componentName)

            MESSAGE( "SALOME_Container_i::create_component_instance : OK")
            comp_o = comp_i._this()
            comp_iors = self._orb.object_to_string(comp_o)
        except:
            ret=traceback.format_exc(10)
            traceback.print_exc()
            MESSAGE( "SALOME_Container_i::create_component_instance : NOT OK")
        return comp_iors, ret
        

    def create_pynode(self,nodeName,code):
        try:
          node=SALOME_PyNode.PyNode_i(nodeName,code,self._poa,self)
          id_o = self._poa.activate_object(node)
          comp_o = self._poa.id_to_reference(id_o)
          comp_iors = self._orb.object_to_string(comp_o)
          return 0,comp_iors
        except:
          exc_typ,exc_val,exc_fr=sys.exc_info()
          l=traceback.format_exception(exc_typ,exc_val,exc_fr)
          return 1,"".join(l)

    def create_pyscriptnode(self,nodeName,code):
        try:
          node=SALOME_PyNode.PyScriptNode_i(nodeName,code,self._poa,self)
          id_o = self._poa.activate_object(node)
          comp_o = self._poa.id_to_reference(id_o)
          comp_iors = self._orb.object_to_string(comp_o)
          return 0,comp_iors
        except:
          exc_typ,exc_val,exc_fr=sys.exc_info()
          l=traceback.format_exception(exc_typ,exc_val,exc_fr)
          return 1,"".join(l)



# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  File   : salome_pynode.py
#  Author : Christian CAREMOLI, EDF
#  Module : SALOME
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/KERNEL_PY/Attic/salome_pynode.py,v 1.1.2.1.14.3.12.1 2012-04-12 14:05:13 vsr Exp $
#
"""
 When imported this module adds to CORBA proxy (from PyNode type) automatic pickle and unpickle
 of arguments and results when calling execute method. It also converts the SALOME exception into a standard python
 exception
"""
import omniORB
import cPickle
import SALOME
import Engines

class SmartPyNode(Engines._objref_PyNode):
  def __init__(self):
    Engines._objref_PyNode.__init__(self)

  def execute(self,functionName,*args,**kws):
    try:
      args=cPickle.dumps((args,kws),-1)
      results=Engines._objref_PyNode.execute(self,functionName,args)
      x=cPickle.loads(results)
      return x
    except SALOME.SALOME_Exception, e:
      raise ValueError(e.details.text)

  def __getattr__(self,name):
    """ a trick to be able to call directly a remote method by its name : no need to use execute"""
    if name[0]== '_':
      raise AttributeError, name
    def afunc(*args,**kws):
      return self.execute(name,*args,**kws)
    return afunc

class SmartPyScriptNode(Engines._objref_PyScriptNode):
  def __init__(self):
    Engines._objref_PyScriptNode.__init__(self)

  def execute(self,outargsname,*args,**kws):
    #the tuple args are ignored
    try:
      args=cPickle.dumps(((),kws),-1)
      results=Engines._objref_PyScriptNode.execute(self,outargsname,args)
      x=cPickle.loads(results)
      return x
    except SALOME.SALOME_Exception, e:
      raise ValueError(e.details.text)

#Register the new proxy for PyNode
omniORB.registerObjref(Engines._objref_PyNode._NP_RepositoryId, SmartPyNode)
#Register the new proxy for PyScriptNode
omniORB.registerObjref(Engines._objref_PyScriptNode._NP_RepositoryId, SmartPyScriptNode)




# -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

# Author: Guillaume Boulant (EDF/R&D)

#
# WARNING: development notes
#
# This python module is aimed to be a (yet another?) end user
# interface to manipulate the KERNEL SALOME services. Note that it
# does not replace the salome module (i.e. what you get when typing
# "import salome", actually implemented in the file __init__.py of the
# salome python package). It provides instead functions that help to
# use the salome module in a end user context (development of domain
# specific tools by programmers that are not ten years experienced in
# SALOME development).

import salome
from deprecation import is_called_by_sphinx
if not is_called_by_sphinx() and salome.lcc is None:
    try:
        salome.salome_init()
    except Exception, e:
        print e

# Note that the salome module provides you with standard SALOME
# objects: CORBA broker (orb): salome.orb lyfe cycle (lcc) :
# salome.lcc naming service : salome.naming_service study manager :
# salome.myStudyManager The default study : salome.myStudy
#
# Alternatively, you may obtain these objects directly with the
# following instructions:
#
#   from omniORB import CORBA
#   from LifeCycleCORBA import LifeCycleCORBA
#   orb = CORBA.ORB_init( [''], CORBA.ORB_ID )
#   lcc = LifeCycleCORBA( orb )
# 
#   from SALOME_NamingServicePy import SALOME_NamingServicePy_i
#   naming_service = SALOME_NamingServicePy_i( orb )
#
# (See salome.py page in the KERNEL documentation)

#
# ==============================================================================
# Helper functions for SALOME components
# ==============================================================================
#

#
# componentName is the name of the component as declared in the XML
# SALOME catalog. A loadable library with name lib<componentName>Engine.so
# is supposed to be reachable. This library is supposed to provide a
# factory function with the prototype:
#
#   PortableServer::ObjectId * <componentName>Engine_factory(
#                                             CORBA::ORB_ptr orb,
#    	                                      PortableServer::POA_ptr poa,
#					      PortableServer::ObjectId * contId,
#					      const char *instanceName,
#					      const char *interfaceName);
#
# corbaModule is the name of the IDL module that contains the
# definition of the interface of the component. This name corresponds
# to the namespace of the servant classes.
#
# containerType specified the container in which the servants are
# executed.
#
def getComponent(componentName = "SalomeTestComponent",
                 corbaModule   = "Engines",
                 containerType = "FactoryServer"):
    """
    Get a SALOME CORBA component from its name
    """
    print "INF: getting component %s from CORBA module %s ..."%(componentName,corbaModule)
    __import__(corbaModule)
    component=salome.lcc.FindOrLoadComponent(containerType,componentName)
    if component is None:
        print "ERR: the SALOME component "+componentName+" can't be reached"
    print "INF: component %s obtained from CORBA module %s"%(componentName,corbaModule)
    return component

# Note that an alternative (and maybe better) method to get a component
# is to use the module catalog. Here, we just use the catalog to get
# the list of components defined in the current session.
import SALOME_ModuleCatalog
def getComponentList():
    """
    Get the list of names of all SALOME componenents register in
    the catalog.
    """
    obj = salome.naming_service.Resolve('Kernel/ModulCatalog')
    catalog = obj._narrow(SALOME_ModuleCatalog.ModuleCatalog)
    if not catalog:
        raise RuntimeError, "Can't accesss module catalog"
    return catalog.GetComponentList()

def getStudyManager():
    """Get a study manager to create and manage SALOME studies"""
    return salome.myStudyManager

import SALOMEDS
def __getStudyManager_demo():
    """
    Get a study manager to create and manage SALOME studies. WARN: you
    should use instead the variable salome.myStudyManager. This
    function is given for illustration of usage of the naming service
    """
    naming_service = SALOME_NamingServicePy_i( orb )
    obj = naming_service.Resolve( '/myStudyManager' )
    studyManager = obj._narrow( SALOMEDS.StudyManager)
    return studyManager


#
# ==============================================================================
# Helper functions for manipulating objects, sobjects and entry
# ==============================================================================
#

# - the SObject is an item in a study (Study Object).
# - the entry is the identifier of an item.
# - the ID is the entry
# - the object (geom object or smesh object) is a CORBA servant
#   embedded in the SALOME component container and with a reference in
#   the SALOME study, so that it can be retrieved.

# __GBO__ WARN: theses functions are already defined in
# salome_study.py, but without the possibility to specify the
# underlying study (in salome_study.py, the study is the default study
# binded to the salome.myStudy attribute). TODO: see if it can be
# extends to the prototype (with the study as an argument) below and
# resorb the functions below.

def IDToObject(id, study=None):
    myObj = None
    if study is None:
        myStudy = salome.myStudy
    else:
        myStudy = study
    mySO = myStudy.FindObjectID(id);
    if mySO is not None:
        ok, anAttr = mySO.FindAttribute("AttributeIOR")
        if ok:
            AtIOR = anAttr._narrow(SALOMEDS.AttributeIOR)
            if AtIOR.Value() != "":
                myObj = salome.orb.string_to_object(AtIOR.Value())
    return myObj

def ObjectToSObject(obj, study=None):
    mySO = None

    if study is None:
        myStudy = salome.myStudy
    else:
        myStudy = study

    if obj is not None:
        ior =  salome.orb.object_to_string(obj)
        if ior != "":
            mySO = myStudy.FindObjectIOR(ior)
    return mySO

def ObjectToID(obj, study=None):
    mySO = ObjectToSObject(obj,study)
    if mySO:
        return mySO.GetID()
    return ""

def IDToSObject(id, study=None):
    if study is None:
        myStudy = salome.myStudy
    else:
        myStudy = study

    mySO = myStudy.FindObjectID(id);
    return mySO

def SObjectToID(sobject):
    if sobject is None: return None
    return sobject.GetID()


#
# ==============================================================================
# Basic use cases and unit tests
# ==============================================================================
#

def TEST_getComponent():
    component=getComponent(componentName = "SalomeTestComponent")
    
    ref_string = 'TestComponent_i : L = 3'
    res_string = component.Coucou(3)
    if ref_string != ref_string:
        return False
    return True

def TEST_getComponentList():
    componentList=getComponentList()
    if 'SalomeTestComponent' not in componentList:
        return False
    return True

def TEST_createObject():
    """
    WARNING: for this test, we need GEOM (used to create an object 
    """
    import geompy
    geompy.init_geom(salome.myStudy)
    box = geompy.MakeBoxDXDYDZ(200, 200, 200)
    id = geompy.addToStudy( box, 'box' )
    return id

def TEST_objectsManipulation():
    myEntry = TEST_createObject()

    mySObject = IDToSObject(myEntry)
    entry     = SObjectToID(mySObject)

    if str(entry) != str(myEntry):
        return False


    myObject = IDToObject(myEntry)
    print myObject
    if myObject is None:
        return False

    return True

if __name__ == "__main__":
    import unittester
    unittester.run("services","TEST_getComponent")
    unittester.run("services","TEST_getComponentList")
    unittester.run("services","TEST_objectsManipulation")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
"""
This module provides a new class :class:`StudyEditor` to complement
:class:`Study` and :class:`StudyBuilder` classes.
"""

import re

import salome
from salome.kernel.logger import Logger
from salome.kernel import termcolor
logger = Logger("salome.kernel.studyedit", color = termcolor.PURPLE)

_editors = {}
_DEFAULT_CONTAINER = "FactoryServer"

def getActiveStudyId():
    """
    Return the ID of the active study. In GUI mode, this function is equivalent
    to ``salome.sg.getActiveStudyId()``. Outside GUI, it returns
    ``salome.myStudyId`` variable.
    """
    salome.salome_init()
    # Warning: we don't use salome.getActiveStudy() here because it doesn't
    # work properly when called from Salome modules (multi-study interpreter
    # issue)
    if salome.hasDesktop():
        return salome.sg.getActiveStudyId()
    else:
        return salome.myStudyId

def getActiveStudy():
    return getStudyFromStudyId(getActiveStudyId())

def getStudyFromStudyId(studyId):
    salome.salome_init()
    study = salome.myStudyManager.GetStudyByID(studyId)
    return study

def getStudyIdFromStudy(study):
    studyId = study._get_StudyId()
    return studyId

def getStudyEditor(studyId = None):
    """
    Return a :class:`StudyEditor` instance to edit the study with ID
    `studyId`. If `studyId` is :const:`None`, return an editor for the current
    study.
    """
    if studyId is None:
        studyId = getActiveStudyId()
    if not _editors.has_key(studyId):
        _editors[studyId] = StudyEditor(studyId)
    return _editors[studyId]

class StudyEditor:
    """
    This class provides utility methods to complement :class:`Study` and
    :class:`StudyBuilder` classes. Those methods may be moved in those classes
    in the future. The parameter `studyId` defines the ID of the study to
    edit. If it is :const:`None`, the edited study will be the current study.
    The preferred way to get a StudyEditor object is through the method
    :meth:`getStudyEditor` which allows to reuse existing instances.

    .. attribute:: studyId
    
       This instance attribute contains the ID of the edited study. This
       attribute should not be modified.

    .. attribute:: study
    
       This instance attribute contains the underlying :class:`Study` object.
       It can be used to access the study but the attribute itself should not
       be modified.

    .. attribute:: builder

       This instance attribute contains the underlying :class:`StudyBuilder`
       object. It can be used to edit the study but the attribute itself
       should not be modified.

    """
    def __init__(self, studyId = None):
        salome.salome_init()
        if studyId is None:
            studyId = getActiveStudyId()
        self.studyId = studyId
        self.study = salome.myStudyManager.GetStudyByID(studyId)
        if self.study is None:
            raise Exception("Can't create StudyEditor object: "
                            "Study %d doesn't exist" % studyId)
        self.builder = self.study.NewBuilder()

    def findOrCreateComponent(self, moduleName, componentName = None,
                              icon = None, containerName = _DEFAULT_CONTAINER):
        """
        Find a component corresponding to the Salome module `moduleName` in
        the study. If none is found, create a new component and associate it
        with the corresponding engine (i.e. the engine named `moduleName`).
        Note that in Salome 5, the module name and engine name must be
        identical (every module must provide an engine with the same name).
        In Salome 6 it will be possible to define a different name for the
        engine.

        :type  moduleName: string
        :param moduleName: name of the module corresponding to the component
                           (the module name is the string value in the
                           attribute "AttributeComment" of the component)

        :type  componentName: string
        :param componentName: name of the new component if created. If
                              :const:`None`, use `moduleName` instead.

        :type  icon: string
        :param icon: icon for the new component (attribute "AttributePixMap").

        :type  containerName: string
        :param containerName: name of the container in which the engine should be
                              loaded.

        :return: the SComponent found or created.

        """
        sComponent = self.study.FindComponent(moduleName)
        if sComponent is None:
            sComponent = self.builder.NewComponent(moduleName)
            # Note that the NewComponent method set the "comment" attribute to the
            # value of its argument (moduleName here)
            if componentName is None:
                componentName = moduleName
            self.builder.SetName(sComponent, componentName)
            if icon is not None:
                # _MEM_ : This will be effective if and only if "moduleName"
                # really corresponds to the module name (as specified in the
                # SalomeApp.xml)
                self.setIcon(sComponent, icon)

            # This part will stay inactive until Salome 6. In Salome 6, the
            # engine name will be stored separately from the module name.
            # An internal convention (in this class) is to store the name of the
            # associated engine in the parameter attribute of the scomponent (so that
            # it could be retrieved in a future usage of this scomponent, for example,
            # for the need of the function loadComponentEngine). The comment attribute
            # SHOULD NOT be used for this purpose  because it's used by the SALOME
            # resources manager to identify the SALOME module and then localized
            # the resource files
            #attr = self.builder.FindOrCreateAttribute( sComponent, "AttributeParameter" )
            #attr.SetString( "ENGINE_NAME", engineName )

            engine = salome.lcc.FindOrLoadComponent(containerName, moduleName)
            if engine is None:
                raise Exception("Cannot load engine %s in container %s. See "
                                "logs of container %s for more details." %
                                (moduleName, containerName, containerName))
            self.builder.DefineComponentInstance(sComponent, engine)

        return sComponent

    def loadComponentEngine(self, sComponent,
                            containerName = _DEFAULT_CONTAINER):
        """
        Load the engine corresponding to `sComponent` in the container
        `containerName`, associate the engine with the component and load the
        CORBA objects of this component in the study.
        """
        # This part will stay inactive until Salome 6. In Salome 6, the
        # engine name will be stored separately from the module name.
        #attr = self.builder.FindOrCreateAttribute( sComponent, "AttributeParameter" )
        #engineName = attr.GetString( "ENGINE_NAME" )
        engine = salome.lcc.FindOrLoadComponent(containerName,
                                                sComponent.GetComment())
        if engine is None:
            raise Exception("Cannot load component %s in container %s. See "
                            "logs of container %s for more details." %
                            (sComponent.GetComment(), containerName,
                             containerName))
        self.builder.LoadWith(sComponent, engine)

    def getOrLoadObject(self, item):
        """
        Get the CORBA object associated with the SObject `item`, eventually by
        first loading it with the corresponding engine.
        """
        object = item.GetObject()
        if object is None: # the engine has not been loaded yet
            sComponent = item.GetFatherComponent()
            self.loadComponentEngine(sComponent)
            object = item.GetObject()
        return object

    def findOrCreateItem(self, fatherItem, name, fileType = None,
                         fileName = None, comment = None, icon = None,
                         IOR = None, typeId = None):
        """
        Find an object under `fatherItem` in the study with the given
        attributes. Return the first one found if at least one exists,
        otherwise create a new one with the given attributes and return it.
        
        See :meth:`setItem` for the description of the parameters.
        """
        sObject = self.findItem(fatherItem, name, fileType, fileName, comment,
                                icon, IOR, typeId)
        if sObject is None:
            sObject = self.createItem(fatherItem, name, fileType, fileName,
                                      comment, icon, IOR, typeId)
        return sObject

    def findItem(self, fatherItem, name = None, fileType = None,
                 fileName = None, comment = None, icon = None, IOR = None,
                 typeId = None):
        """
        Find an item with given attributes under `fatherItem` in the study. If
        none is found, return :const:`None`. If several items correspond to
        the parameters, only the first one is returned. The search is made
        only on given parameters (i.e. not :const:`None`). To look explicitly
        for an empty attribute, use an empty string in the corresponding
        parameter.
        
        See :meth:`setItem` for the description of the parameters.
        """
        foundItem = None;
        childIterator = self.study.NewChildIterator(fatherItem)
        while childIterator.More() and foundItem is None:
            childItem = childIterator.Value()
            if childItem and \
               (name is None or childItem.GetName() == name) and \
               (fileType is None or \
                self.getFileType(childItem) == fileType) and \
               (fileName is None or \
                self.getFileName(childItem) == fileName) and \
               (comment is None or childItem.GetComment() == comment) and \
               (icon is None or \
                self.getIcon(childItem) == icon) and \
               (IOR is None or childItem.GetIOR() == IOR) and \
               (typeId is None or \
                self.getTypeId(childItem) == typeId):
                foundItem = childItem
            childIterator.Next()
        return foundItem

    def createItem(self, fatherItem, name, fileType = None, fileName = None,
                   comment = None, icon = None, IOR = None, typeId = None):
        """
        Create a new object named `name` under `fatherItem` in the study, with
        the given attributes. If an object named `name` already exists under
        the father object, the new object is created with a new name `name_X`
        where X is the first available index.
        
        :type  fatherItem: SObject
        :param fatherItem: item under which the new item will be added.
                
        :return: new SObject created in the study
        
        See :meth:`setItem` for the description of the other parameters.
        """
        aSObject = self.builder.NewObject(fatherItem)

        aChildIterator = self.study.NewChildIterator(fatherItem)
        aMaxId = -1
        aLength = len(name)
        aDelim = "_"
        anIdRE = re.compile("^" + aDelim + "[0-9]+")
        aNameRE = re.compile("^" + name)
        while aChildIterator.More():
            aSObj = aChildIterator.Value()
            aChildIterator.Next()
            aName = aSObj.GetName()
            if re.match(aNameRE,aName):
                aTmp = aName[aLength:]
                if re.match(anIdRE,aTmp):
                    import string
                    anId = string.atol(aTmp[1:])
                    if aMaxId < anId:
                        aMaxId = anId
                        pass
                    pass
                elif aMaxId < 0:
                    aMaxId = 0
                    pass
                pass
            pass
        
        aMaxId = aMaxId + 1
        aName = name
        if aMaxId > 0:
            aName = aName + aDelim + str(aMaxId)
            pass
        
        self.setItem(aSObject, aName, fileType, fileName, comment, icon,
                     IOR, typeId)
    
        return aSObject

    def setItem(self, item, name = None, fileType = None, fileName = None,
                comment = None, icon = None, IOR = None, typeId = None):
        """
        Modify the attributes of an item in the study. Unspecified attributes
        (i.e. those set to :const:`None`) are left unchanged.

        :type  item: SObject
        :param item: item to modify.

        :type  name: string
        :param name: item name (attribute 'AttributeName').

        :type  fileType: string
        :param fileType: item file type (attribute 'AttributeFileType').

        :type  fileName: string
        :param fileName: item file name (attribute
                         'AttributeExternalFileDef').

        :type  comment: string
        :param comment: item comment (attribute 'AttributeComment'). Note that
                        this attribute will appear in the 'Value' column in
                        the object browser.

        :type  icon: string
        :param icon: item icon name (attribute 'AttributePixMap').

        :type  IOR: string
        :param IOR: IOR of a CORBA object associated with the item
                    (attribute 'AttributeIOR').

        :type  typeId: integer
        :param typeId: item type (attribute 'AttributeLocalID').
        """
        logger.debug("setItem (ID=%s): name=%s, fileType=%s, fileName=%s, "
                     "comment=%s, icon=%s, IOR=%s" %
                     (item.GetID(), name, fileType, fileName, comment,
                      icon, IOR))
        # Explicit cast is necessary for unicode to string conversion
        if name is not None:
            self.builder.SetName(item, str(name))
        if fileType is not None:
            self.setFileType(item, fileType)
        if fileName is not None:
            self.setFileName(item, fileName)
        if comment is not None:
            self.builder.SetComment(item, str(comment))
        if icon is not None:
            self.setIcon(item, icon)
        if IOR is not None:
            self.builder.SetIOR(item, str(IOR))
        if typeId is not None:
            self.setTypeId(item, typeId)

    def removeItem(self, item, withChildren = False ):
        """
        Remove the given item from the study. Note that the items are never
        really deleted. They just don't appear in the study anymore.

        :type  item: SObject
        :param item: the item to be removed

        :type  withChildren: boolean
        :param withChildren: if :const:`True`, also remove the children of
                             `item`

        :return: :const:`True` if the item was removed successfully, or
                 :const:`False` if an error happened.
        """
        ok = False
        try:
            if withChildren:
                self.builder.RemoveObjectWithChildren(item)
            else:
                self.builder.RemoveObject(item)
            ok = True
        except:
            ok = False
        return ok

    def setItemAtTag(self, fatherItem, tag, name = None, fileType = None,
                     fileName = None, comment = None, icon = None, IOR = None,
                     typeId = None):
        """
        Find an item tagged `tag` under `fatherItem` in the study tree or
        create it if there is none, then set its attributes.
        
        :type  fatherItem: SObject
        :param fatherItem: item under which the tagged item will be looked for
                           and eventually created.

        :type  tag: integer
        :param tag: tag of the item to look for.

        :return: the SObject at `tag` if found or created successfully, or
                 :const:`None` if an error happened.
        
        See :meth:`setItem` for the description of the other parameters.
        """
        found, sObj = fatherItem.FindSubObject(tag)
        if not found:
            sObj = self.builder.NewObjectToTag(fatherItem, tag)
        self.setItem(sObj, name, fileType, fileName, comment, icon,
                     IOR, typeId)
        return sObj

    def getAttributeValue(self, sObject, attributeName, default = None):
        """
        Return the value of the attribute named `attributeName` on the object
        `sObject`, or `default` if the attribute doesn't exist.
        """
        value = default
        found, attr = self.builder.FindAttribute(sObject, attributeName)
        if found:
            value = attr.Value()
        return value

    def setAttributeValue(self, sObject, attributeName, attributeValue):
        """
        Set the value of the attribute named `attributeName` on the object
        `sObject` to the value `attributeValue`.
        """        
        attr = self.builder.FindOrCreateAttribute(sObject, attributeName)
        attr.SetValue(attributeValue)

    def getTypeId(self, sObject):
        """
        Return the value of the attribute "AttributeLocalID" of the object
        `sObject`, or :const:`None` if it is not set.
        """
        return self.getAttributeValue(sObject, "AttributeLocalID")

    def setTypeId(self, sObject, value):
        """
        Set the attribute "AttributeLocalID" of the object `sObject` to the
        value `value`.
        """
        self.setAttributeValue(sObject, "AttributeLocalID", value)

    def getFileType(self, sObject):
        """
        Return the value of the attribute "AttributeFileType" of the object
        `sObject`, or an empty string if it is not set.
        """
        return self.getAttributeValue(sObject, "AttributeFileType", "")

    def setFileType(self, sObject, value):
        """
        Set the attribute "AttributeFileType" of the object `sObject` to the
        value `value`.
        """
        # Explicit cast is necessary for unicode to string conversion
        self.setAttributeValue(sObject, "AttributeFileType", str(value))

    def getFileName(self, sObject):
        """
        Return the value of the attribute "AttributeExternalFileDef" of the
        object `sObject`, or an empty string if it is not set.
        """
        return self.getAttributeValue(sObject, "AttributeExternalFileDef", "")

    def setFileName(self, sObject, value):
        """
        Set the attribute "AttributeExternalFileDef" of the object `sObject`
        to the value `value`.
        """
        # Explicit cast is necessary for unicode to string conversion
        self.setAttributeValue(sObject, "AttributeExternalFileDef",
                               str(value))

    def getIcon(self, sObject):
        """
        Return the value of the attribute "AttributePixMap" of the object
        `sObject`, or an empty string if it is not set.
        """
        value = ""
        found, attr = self.builder.FindAttribute(sObject, "AttributePixMap")
        if found and attr.HasPixMap():
            value = attr.GetPixMap()
        return value

    def setIcon(self, sObject, value):
        """
        Set the attribute "AttributePixMap" of the object `sObject` to the
        value `value`.
        """
        attr = self.builder.FindOrCreateAttribute(sObject, "AttributePixMap")
        # Explicit cast is necessary for unicode to string conversion
        attr.SetPixMap(str(value))



#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

# ---
# File   : salome_utils.py
# Author : Vadim SANDLER, Open CASCADE S.A.S. (vadim.sandler@opencascade.com)
# ---

## @package salome_utils
# \brief Set of utility functions used by SALOME python scripts.

#
# Exported functions
#

__all__ = [
    'getORBcfgInfo',
    'getHostFromORBcfg',
    'getPortFromORBcfg',
    'getUserName',
    'getHostName',
    'getShortHostName',
    'getAppName',
    'getPortNumber',
    'getTmpDir',
    'getHomeDir',
    'generateFileName',
    'makeTmpDir',
    'uniteFiles',
    ]

# ---

def _try_bool( arg ):
    """
    Check if specified parameter represents boolean value and returns its value.
    String values like 'True', 'TRUE', 'YES', 'Yes', 'y', 'NO', 'false', 'n', etc
    are supported.
    If <arg> does not represent a boolean, an exception is raised.
    """
    import types
    if type( arg ) == types.BooleanType  :
        return arg
    elif type( arg ) == types.StringType  :
        v = str( arg ).lower()
        if   v in [ "yes", "y", "true"  ]: return True
        elif v in [ "no",  "n", "false" ]: return False
        pass
    raise Exception("Not boolean value")

# ---

def getORBcfgInfo():
    """
    Get omniORB current configuration.
    Returns a list of three values: [ orb_version, host_name, port_number ].
    
    The information is retrieved from the omniORB configuration file defined
    by the OMNIORB_CONFIG environment variable.
    If omniORB configuration file can not be accessed, a list of three empty
    strings is returned.
    """
    import os, re
    ret = [ "", "", "" ]
    try:
        f = open( os.getenv( "OMNIORB_CONFIG" ) )
        lines = f.readlines()
        f.close()
        regvar = re.compile( "(ORB)?InitRef.*corbaname::(.*):(\d+)\s*$" )
        for l in lines:
            try:
                m = regvar.match( l )
                if m:
                    if m.group(1) is None:
                        ret[0] = "4"
                    else:
                        ret[0] = "3"
                        pass
                    ret[1] = m.group(2)
                    ret[2] = m.group(3)
                    break
                pass
            except:
                pass
            pass
        pass
    except:
        pass
    return ret

# ---

def getHostFromORBcfg():
    """
    Get current omniORB host.
    """
    return getORBcfgInfo()[1]
# ---

def getPortFromORBcfg():
    """
    Get current omniORB port.
    """
    return getORBcfgInfo()[2]

# ---

def getUserName():
    """
    Get user name:
    1. try USER environment variable
    2. if fails, return 'unknown' as default user name
    """
    import os
    return os.getenv( "USER", "unknown" ) # 'unknown' is default user name

# ---

def getHostName():
    """
    Get host name:
    1. try socket python module gethostname() function
    2. if fails, try HOSTNAME environment variable
    3. if fails, try HOST environment variable
    4. if fails, return 'unknown' as default host name
    """
    import os
    try:
        import socket
        host = socket.gethostname()
    except:
        host = None
        pass
    if not host: host = os.getenv("HOSTNAME")
    if not host: host = os.getenv("HOST")
    if not host: host = "unknown"           # 'unknown' is default host name
    return host

# ---

def getShortHostName():
    """
    Get short host name:
    1. try socket python module gethostname() function
    2. if fails, try HOSTNAME environment variable
    3. if fails, try HOST environment variable
    4. if fails, return 'unknown' as default host name
    """
    try:
        return getHostName().split('.')[0]
    except:
        pass
    return "unknown"           # 'unknown' is default host name
    
# ---

def getAppName():
    """
    Get application name:
    1. try APPNAME environment variable
    2. if fails, return 'SALOME' as default application name
    """
    import os
    return os.getenv( "APPNAME", "SALOME" ) # 'SALOME' is default user name

# ---

def getPortNumber(use_default=True):
    """
    Get current naming server port number:
    1. try NSPORT environment variable
    1. if fails, try to parse config file defined by OMNIORB_CONFIG environment variable
    2. if fails, return 2809 as default port number (if use_default is True) or None (id use_default is False)
    """
    import os
    try:
        return int( os.getenv( "NSPORT" ) )
    except:
        pass
    port = getPortFromORBcfg()
    if port is not None: return port
    if use_default: return 2809 # '2809' is default port number
    return None

# ---

def getHomeDir():
    """
    Get home directory.
    """
    import os, sys
    if sys.platform == "win32":
        # for Windows the home directory is detected in the following way:
        # 1. try USERPROFILE env variable
        # 2. try combination of HOMEDRIVE and HOMEPATH env variables
        # 3. try HOME env variable
        # TODO: on Windows, also GetUserProfileDirectoryW() system function might be used
        dir = os.getenv("USERPROFILE")
        if not dir and os.getenv("HOMEDRIVE") and os.getenv("HOMEPATH"):
            dir = os.path.join(os.getenv("HOMEDRIVE"), os.getenv("HOMEPATH"))
        if not dir:
            dir = os.getenv("HOME")
        pass
    else:
        # for Linux: use HOME variable
        dir = os.getenv("HOME")
        pass
    return dir
# ---

def getTmpDir():
    """
    Get directory to be used for the temporary files.
    """
    import os, sys
    if sys.platform == "win32":
        # for Windows: temporarily using home directory for tmp files;
        # to be replaced with TEMP environment variable later...
        dir = os.getenv("HOME")
    else:
        # for Linux: use /tmp/logs/{user} folder
        dir = os.path.join( '/tmp', 'logs', getUserName() )
        pass
    return dir

# ---

def generateFileName( dir, prefix = None, suffix = None, extension = None,
                      unique = False, separator = "_", hidden = False, **kwargs ):
    """
    Generate file name by sepecified parameters. If necessary, file name
    can be generated to be unique.

    Parameters:
    - dir       : directory path
    - prefix    : file prefix (not added by default)
    - suffix    : file suffix (not added by default)
    - extension : file extension (not added by default)
    - unique    : if this parameter is True, the unique file name is generated:
    in this case, if the file with the generated name already exists
    in the <dir> directory, an integer suffix is added to the end of the
    file name. This parameter is False by default.
    - separator : separator of the words ('_' by default)
    - hidden    : if this parameter is True, the file name is prepended by . (dot)
    symbol. This parameter is False by default.

    Other keyword parameters are:
    - with_username : 'add user name' flag/option:
      * boolean value can be passed to determine user name automatically
      * string value to be used as user name
    - with_hostname : 'add host name' flag/option:
      * boolean value can be passed to determine host name automatically
      * string value to be used as host name
    - with_port     : 'add port number' flag/option:
      * boolean value can be passed to determine port number automatically
      * string value to be used as port number
    - with_app      : 'add application name' flag/option:
      * boolean value can be passed to determine application name automatically
      * string value to be used as application name
    All <with_...> parameters are optional.
    """
    supported = [ 'with_username', 'with_hostname', 'with_port', 'with_app' ]
    from launchConfigureParser import verbose
    filename = []
    # separator
    if separator is None:
        separator = ""
        pass
    else:
        separator = str( separator )
        pass
    # prefix (if specified)
    if prefix is not None:
        filename.append( str( prefix ) )
        pass
    # additional keywords
    ### check unsupported parameters
    for kw in kwargs:
        if kw not in supported and verbose():
            print 'Warning! salome_utilitie.py: generateFileName(): parameter %s is not supported' % kw
            pass
        pass
    ### process supported keywords
    for kw in supported:
        if kw not in kwargs: continue
        ### user name
        if kw == 'with_username':
            try:
                # auto user name ?
                if _try_bool( kwargs[kw] ): filename.append( getUserName() )
                pass
            except:
                # user name given as parameter
                filename.append( kwargs[kw] )
                pass
            pass
        ### host name
        elif kw == 'with_hostname':
            try:
                # auto host name ?
                if _try_bool( kwargs[kw] ): filename.append( getShortHostName() )
                pass
            except:
                # host name given as parameter
                filename.append( kwargs[kw] )
                pass
            pass
        ### port number
        elif kw == 'with_port':
            try:
                # auto port number ?
                if _try_bool( kwargs[kw] ): filename.append( str( getPortNumber() ) )
                pass
            except:
                # port number given as parameter
                filename.append( str( kwargs[kw] ) )
                pass
            pass
        ### application name
        elif kw == 'with_app':
            try:
                # auto application name ?
                if _try_bool( kwargs[kw] ): filename.append( getAppName() )
                pass
            except:
                # application name given as parameter
                filename.append( kwargs[kw] )
                pass
            pass
        pass
    # suffix (if specified)
    if suffix is not None:
        filename.append( str( suffix ) )
        pass
    # raise an exception if file name is empty
    if not filename:
        raise Exception("Empty file name")
    #
    if extension is not None and extension.startswith("."): extension = extension[1:]
    #
    import os
    name = separator.join( filename )
    if hidden: name = "." + name                       # add dot for hidden files
    if extension: name = name + "." + str( extension ) # add extension if defined
    name = os.path.join( dir, name )
    if unique:
        # create unique file name
        index = 0
        while os.path.exists( name ):
            index = index + 1
            name = separator.join( filename ) + separator + str( index )
            if hidden: name = "." + name                       # add dot for hidden files
            if extension: name = name + "." + str( extension ) # add extension if defined
            name = os.path.join( dir, name )
            pass
        pass
    return os.path.normpath(name)

# ---

def makeTmpDir( path, mode=0777 ):
    """
    Make temporary directory with the specified path.
    If the directory exists then clear its contents.

    Parameters:
    - path : absolute path to the directory to be created.
    - mode : access mode
    """
    import os
    if os.path.exists( path ):
        import sys
        if sys.platform == "win32":
            os.system( "rmdir /S /Q " + '"' + path + '"' )
            os.system( "mkdir " + '"' + path + '"' )
        else:
            os.system( "rm -rf " + path + "/*" )
    else:
        dirs = path.split("/")
        shift1 = shift2 = 0
        if not dirs[0]: shift1 = 1
        if dirs[-1]: shift2 = 1
        for i in range(1+shift1,len(dirs)+shift2):
            p = "/".join(dirs[:i])
            try:
                os.mkdir(p, mode)
                os.chmod(p, mode)
            except:
                pass

# ---

def uniteFiles( src_file, dest_file ):
    """
    Unite contents of the source file with contents of the destination file
    and put result of the uniting to the destination file.
    If the destination file does not exist then the source file is simply
    copied to its path.

    Parameters:
    - src_file  : absolute path to the source file
    - dest_file : absolute path to the destination file
    """
    import os

    if not os.path.exists( src_file ):
        return
        pass

    if os.path.exists( dest_file ):
        # add a symbol of new line to contents of the destination file (just in case)
        dest = open( dest_file, 'r' )
        dest_lines = dest.readlines()
        dest.close()

        dest_lines.append( "\n" )

        dest = open( dest_file, 'w' )
        dest.writelines( dest_lines )
        dest.close()

        import sys
        if sys.platform == "win32":
            command = "type " + '"' + src_file + '"' + " >> " + '"' + dest_file + '"'
        else:
            command = "cat " + src_file + " >> " + dest_file
            pass
        pass
    else:
        import sys
        if sys.platform == "win32":
            command = "copy " + '"' + src_file + '"' + " " + '"' + dest_file + '"' + " > nul"
        else:
            command = "cp " + src_file + " " + dest_file
            pass
        pass

    os.system( command )

# --

_verbose = None

def verbose():
    """
    Get verbosity level. Default verbosity level is specified via the environment variable
    SALOME_VERBOSE, e.g.:
    [bash %] export SALOME_VERBOSE=1
    The function setVerbose() can be used to change verbosity level explicitly.
    """
    global _verbose
    # verbose has already been called
    if _verbose is not None:
        return _verbose
    # first time
    try:
        from os import getenv
        _verbose = int(getenv('SALOME_VERBOSE'))
    except:
        _verbose = 0
        pass
    #
    return _verbose

def setVerbose(level):
    """
    Change verbosity level. The function verbose() can be used to get current verbosity level.
    """
    global _verbose
    _verbose = level
    return




#!/usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

## @package runSalome
# \brief Module that provides services to launch SALOME
#

import sys, os, string, glob, time, pickle, re
import orbmodule
import setenv
from launchConfigureParser import verbose
from server import process_id, Server

if sys.platform == "win32":
    SEP = ";"
else:
    SEP = ":"

# -----------------------------------------------------------------------------

from killSalome import killAllPorts

def killLocalPort():
    """
    kill servers from a previous SALOME exection, if needed,
    on the CORBA port given in args of runSalome
    """
    
    from killSalomeWithPort import killMyPort
    my_port=str(args['port'])
    try:
        killMyPort(my_port)
    except:
        print "problem in killLocalPort()"
        pass
    pass
    
def givenPortKill(port):
    """
    kill servers from a previous SALOME exection, if needed,
    on the same CORBA port
    """
    
    from killSalomeWithPort import killMyPort
    my_port=port
    try:
        killMyPort(my_port)
    except:
        print "problem in LocalPortKill(), killMyPort("<<port<<")"
        pass
    pass

def kill_salome(args):
    """
    Kill servers from previous SALOME executions, if needed;
    depending on args 'killall' or 'portkill', kill all executions,
    or only execution on the same CORBA port
    """

    if args['killall']:
        killAllPorts()
    elif args['portkill']:
        givenPortKill(str(args['port']))

# -----------------------------------------------------------------------------
#
# Class definitions to launch CORBA Servers
#

class InterpServer(Server):
    def __init__(self,args):
        self.args=args
        if sys.platform != "win32":
          env_ld_library_path=['env', 'LD_LIBRARY_PATH=' + os.getenv("LD_LIBRARY_PATH")]
          self.CMD=['xterm', '-e']+ env_ld_library_path + ['python']
        else:
          self.CMD=['cmd', '/c', 'start cmd.exe', '/K', 'python']
       
    def run(self):
        global process_id
        command = self.CMD
        print "INTERPSERVER::command = ", command
        if sys.platform == "win32":
          import win32pm
          pid = win32pm.spawnpid( string.join(command, " "),'-nc' )
        else:
          pid = os.spawnvp(os.P_NOWAIT, command[0], command)
        process_id[pid]=self.CMD
        self.PID = pid

# ---

def get_cata_path(list_modules,modules_root_dir):
    """Build a list of catalog paths (cata_path) to initialize the ModuleCatalog server
    """
    modules_cata={}
    cata_path=[]

    for module in list_modules:
        if modules_root_dir.has_key(module):
            module_root_dir=modules_root_dir[module]
            module_cata=module+"Catalog.xml"
            cata_file=os.path.join(module_root_dir, "share",setenv.salome_subdir, "resources",module.lower(), module_cata)

            if os.path.exists(cata_file):
                cata_path.append(cata_file)
                modules_cata[module]=cata_file
            else:
                cata_file=os.path.join(module_root_dir, "share",setenv.salome_subdir, "resources", module_cata)
                if os.path.exists(cata_file):
                    cata_path.append(cata_file)
                    modules_cata[module]=cata_file

    for path in os.getenv("SALOME_CATALOGS_PATH","").split(SEP):
        if os.path.exists(path):
            for cata_file in glob.glob(os.path.join(path,"*Catalog.xml")):
                module_name= os.path.basename(cata_file)[:-11]
                if not modules_cata.has_key(module_name):
                    cata_path.append(cata_file)
                    modules_cata[module_name]=cata_file

    return cata_path



class CatalogServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.SCMD1=['SALOME_ModuleCatalog_Server','-common']
        self.SCMD2=[]
        home_dir=os.getenv('HOME')
        if home_dir is not None:
            self.SCMD2=['-personal',os.path.join(home_dir,'Salome/resources/CatalogModulePersonnel.xml')] 

    def setpath(self,modules_list,modules_root_dir):
        list_modules = modules_list[:]
        list_modules.reverse()
        if self.args["gui"] :
            list_modules = ["KERNEL", "GUI"] + list_modules
        else :
            list_modules = ["KERNEL"] + list_modules

        cata_path=get_cata_path(list_modules,modules_root_dir)

        self.CMD=self.SCMD1 + ['"' + string.join(cata_path,'"::"') + '"'] + self.SCMD2

# ---

class SalomeDSServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.CMD=['SALOMEDS_Server']

# ---

class ConnectionManagerServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.CMD=['SALOME_ConnectionManagerServer']

# ---

class RegistryServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.CMD=['SALOME_Registry_Server', '--salome_session','theSession']

# ---

class ContainerCPPServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.CMD=['SALOME_Container','FactoryServer']

# ---

class ContainerPYServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        if sys.platform == "win32":
          self.CMD=[os.environ["PYTHONBIN"], '\"'+os.environ["KERNEL_ROOT_DIR"] + '/bin/salome/SALOME_ContainerPy.py'+'\"','FactoryServerPy']
        else:
          self.CMD=['SALOME_ContainerPy.py','FactoryServerPy']

# ---

class LoggerServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        from salome_utils import generateFileName
        if sys.platform == "win32": dirpath = os.environ["HOME"]
        else:                       dirpath = "/tmp"
        logfile = generateFileName( dirpath,
                                    prefix="logger",
                                    extension="log",
                                    with_username=True,
                                    with_hostname=True,
                                    with_port=True)
        print "==========================================================="
        print "Logger server: put log to the file:"
        print logfile
        print "==========================================================="
        self.CMD=['SALOME_Logger_Server', logfile]
        pass
    pass # end of LoggerServer class

# ---

class SessionServer(Server):
    def __init__(self,args,modules_list,modules_root_dir):
        self.args = args.copy()
        # Bug 11512 (Problems with runSalome --xterm on Mandrake and Debian Sarge)
        #self.args['xterm']=0
        #
        self.initArgs()
        self.SCMD1=['SALOME_Session_Server']
        self.SCMD2=[]
        if 'registry' in self.args['embedded']:
            self.SCMD1+=['--with','Registry',
                         '(','--salome_session','theSession',')']
        if 'moduleCatalog' in self.args['embedded']:
            self.SCMD1+=['--with','ModuleCatalog','(','-common']
            home_dir=os.getenv('HOME')
            if home_dir is not None:
                self.SCMD2+=['-personal',os.path.join(home_dir,'Salome/resources/CatalogModulePersonnel.xml')] 
            self.SCMD2+=[')']
        if 'study' in self.args['embedded']:
            self.SCMD2+=['--with','SALOMEDS','(',')']
        if 'cppContainer' in self.args['embedded']:
            self.SCMD2+=['--with','Container','(','FactoryServer',')']
        if 'SalomeAppEngine' in self.args['embedded']:
            self.SCMD2+=['--with','SalomeAppEngine','(',')']
            
        if 'cppContainer' in self.args['standalone'] or 'cppContainer' in self.args['embedded']:
            self.SCMD2+=['CPP']
        if 'pyContainer' in self.args['standalone'] or 'pyContainer' in self.args['embedded']:
            self.SCMD2+=['PY']
        if self.args['gui']:
            session_gui = True
            if self.args.has_key('session_gui'):
                session_gui = self.args['session_gui']
            if session_gui:
                self.SCMD2+=['GUI']
                if self.args['splash']:
                    self.SCMD2+=['SPLASH']
                    pass
                if self.args['study_hdf'] is not None:
                    self.SCMD2+=['--study-hdf=%s'%self.args['study_hdf']]
                    pass
                pass
            pass
        if self.args['noexcepthandler']:
            self.SCMD2+=['noexcepthandler']
        if self.args.has_key('user_config'):
            self.SCMD2+=['--resources=%s'%self.args['user_config']]
        if self.args.has_key('modules'):
            list_modules = []
            #keep only modules with GUI
            for m in modules_list:
              if m not in modules_root_dir:
                list_modules.insert(0,m)
              else:
                fr1 = os.path.join(modules_root_dir[m],"share","salome","resources",m.lower(),"SalomeApp.xml")
                fr2 = os.path.join(modules_root_dir[m],"share","salome","resources","SalomeApp.xml")
                if os.path.exists(fr1) or os.path.exists(fr2):
                  list_modules.insert(0,m)
            list_modules.reverse()
            self.SCMD2+=['--modules (%s)' % ":".join(list_modules)]

        if self.args.has_key('pyscript') and len(self.args['pyscript']) > 0:
            self.SCMD2+=['--pyscript=%s'%(",".join(self.args['pyscript']))]

    def setpath(self,modules_list,modules_root_dir):
        list_modules = modules_list[:]
        list_modules.reverse()
        if self.args["gui"] :
            list_modules = ["KERNEL", "GUI"] + list_modules
        else :
            list_modules = ["KERNEL"] + list_modules

        cata_path=get_cata_path(list_modules,modules_root_dir)

        if (self.args["gui"]) & ('moduleCatalog' in self.args['embedded']):
            #Use '::' instead ":" because drive path with "D:\" is invalid on windows platform
            self.CMD=self.SCMD1 + ['"' + string.join(cata_path,'"::"') + '"'] + self.SCMD2
        else:
            self.CMD=self.SCMD1 + self.SCMD2
        if self.args.has_key('test'):
            self.CMD+=['-test'] + self.args['test']
        elif self.args.has_key('play'):
            self.CMD+=['-play'] + self.args['play']

        if self.args["gdb_session"] or self.args["ddd_session"]:
            f = open(".gdbinit4salome", "w")
            f.write("set args ")
            args = " ".join(self.CMD[1:])
            args = args.replace("(", "\(")
            args = args.replace(")", "\)")
            f.write(args)
            f.write("\n")
            f.close()
            if self.args["ddd_session"]:
                self.CMD = ["ddd", "--command=.gdbinit4salome", self.CMD[0]]
            elif self.args["gdb_session"]:
                self.CMD = ["xterm", "-e", "gdb", "--command=.gdbinit4salome", self.CMD[0]]
                pass
            pass
        
        if self.args["valgrind_session"]:
            l = ["valgrind"]
            val = os.getenv("VALGRIND_OPTIONS")
            if val:
                l += val.split()
                pass
            self.CMD = l + self.CMD
            pass
        
# ---

class LauncherServer(Server):
    def __init__(self,args):
        self.args=args
        self.initArgs()
        self.SCMD1=['SALOME_LauncherServer']
        self.SCMD2=[]
        if args["gui"] :
            if 'registry' in self.args['embedded']:
                self.SCMD1+=['--with','Registry',
                             '(','--salome_session','theSession',')']
            if 'moduleCatalog' in self.args['embedded']:
                self.SCMD1+=['--with','ModuleCatalog','(','-common']
                self.SCMD2+=['-personal',
                             '${HOME}/Salome/resources/CatalogModulePersonnel.xml',')']
            if 'study' in self.args['embedded']:
                self.SCMD2+=['--with','SALOMEDS','(',')']
            if 'cppContainer' in self.args['embedded']:
                self.SCMD2+=['--with','Container','(','FactoryServer',')']

    def setpath(self,modules_list,modules_root_dir):
        list_modules = modules_list[:]
        list_modules.reverse()
        if self.args["gui"] :
            list_modules = ["KERNEL", "GUI"] + list_modules
        else :
            list_modules = ["KERNEL"] + list_modules

        cata_path=get_cata_path(list_modules,modules_root_dir)

        if (self.args["gui"]) & ('moduleCatalog' in self.args['embedded']):
            #Use '::' instead ":" because drive path with "D:\" is invalid on windows platform
            self.CMD=self.SCMD1 + ['"' + string.join(cata_path,'"::"') + '"'] + self.SCMD2
        else:
            self.CMD=self.SCMD1 + self.SCMD2

class NotifyServer(Server):
    def __init__(self,args,modules_root_dir):
        self.args=args
        self.initArgs()
        self.modules_root_dir=modules_root_dir
        myLogName = os.environ["LOGNAME"]
        self.CMD=['notifd','-c',
                  self.modules_root_dir["KERNEL"] +'/share/salome/resources/kernel/channel.cfg',
                  '-DFactoryIORFileName=/tmp/'+myLogName+'_rdifact.ior',
                  '-DChannelIORFileName=/tmp/'+myLogName+'_rdichan.ior',
                  '-DReportLogFile=/tmp/'+myLogName+'_notifd.report',
                  '-DDebugLogFile=/tmp/'+myLogName+'_notifd.debug',
                  ]

#
# -----------------------------------------------------------------------------

def startGUI(clt):
    """Salome Session Graphic User Interface activation"""
    import Engines
    import SALOME
    import SALOMEDS
    import SALOME_ModuleCatalog
    import SALOME_Session_idl
    session=clt.waitNS("/Kernel/Session",SALOME.Session)
    session.GetInterface()
  
# -----------------------------------------------------------------------------

def startSalome(args, modules_list, modules_root_dir):
    """Launch all SALOME servers requested by args"""
    init_time = os.times()

    if verbose(): print "startSalome ", args
    
    #
    # Set server launch command
    #
    if args.has_key('server_launch_mode'):
        Server.set_server_launch_mode(args['server_launch_mode'])
    
    #
    # Wake up session option
    #
    if args['wake_up_session']:
        if "OMNIORB_CONFIG" not in os.environ:
            from salome_utils import generateFileName
            home  = os.getenv("HOME")
            appli = os.getenv("APPLI")
            kwargs={}
            if appli is not None: 
                home = os.path.join(home, appli,"USERS")
                kwargs["with_username"] = True
                pass
            last_running_config = generateFileName(home, prefix="omniORB",
                                                   suffix="last",
                                                   extension="cfg",
                                                   hidden=True,
                                                   **kwargs)
            os.environ['OMNIORB_CONFIG'] = last_running_config
            pass
        pass
    
    #
    # Initialisation ORB and Naming Service
    #
   
    clt=orbmodule.client(args)

    #
    # Wake up session option
    #
    if args['wake_up_session']:
        import Engines
        import SALOME
        import SALOMEDS
        import SALOME_ModuleCatalog
        import SALOME_Session_idl
        session = clt.waitNS("/Kernel/Session",SALOME.Session)
        status = session.GetStatSession()
        if status.activeGUI:
            from salome_utils import getPortNumber
            port = getPortNumber()
            msg  = "Warning :"
            msg += "\n"
            msg += "Session GUI for port number %s is already active."%(port)
            msg += "\n"
            msg += "If you which to wake up another session,"
            msg += "\n"
            msg += "please use variable OMNIORB_CONFIG"
            msg += "\n"
            msg += "to get the correct session object in naming service."
            sys.stdout.write(msg+"\n")
            sys.stdout.flush()
            return clt
        session.GetInterface()
        args["session_object"] = session
        return clt
    
    # Save Naming service port name into
    # the file args["ns_port_log_file"]
    if args.has_key('ns_port_log_file'):
      home = os.environ['HOME']
      appli= os.environ.get("APPLI")
      if appli is not None:
        home = os.path.join(home, appli, "USERS")
      file_name = os.path.join(home, args["ns_port_log_file"])
      f = open(file_name, "w")
      f.write(os.environ['NSPORT'])
      f.close()

    # Launch Logger Server (optional)
    # and wait until it is registered in naming service
    #

    if args['logger']:
        myServer=LoggerServer(args)
        myServer.run()
        clt.waitLogger("Logger")

    # Notify Server launch
    #

    if sys.platform != "win32":
      if verbose(): print "Notify Server to launch"
    
      myServer=NotifyServer(args,modules_root_dir)
      myServer.run()

    # Launch  Session Server (to show splash ASAP)
    #

    if args["gui"]:
        mySessionServ = SessionServer(args,args['modules'],modules_root_dir)
        mySessionServ.setpath(modules_list,modules_root_dir)
        mySessionServ.run()

    #
    # Launch Registry Server,
    # and wait until it is registered in naming service
    #

    if ('registry' not in args['embedded']) | (args["gui"] == 0) :
        myServer=RegistryServer(args)
        myServer.run()
        if sys.platform == "win32":
          clt.waitNS("/Registry")
        else:
          clt.waitNSPID("/Registry",myServer.PID)

    #
    # Launch Catalog Server,
    # and wait until it is registered in naming service
    #

    if ('moduleCatalog' not in args['embedded']) | (args["gui"] == 0):
        cataServer=CatalogServer(args)
        cataServer.setpath(modules_list,modules_root_dir)
        cataServer.run()
        import SALOME_ModuleCatalog
        if sys.platform == "win32":
          clt.waitNS("/Kernel/ModulCatalog",SALOME_ModuleCatalog.ModuleCatalog)
        else:
          clt.waitNSPID("/Kernel/ModulCatalog",cataServer.PID,SALOME_ModuleCatalog.ModuleCatalog)

    #
    # Launch SalomeDS Server,
    # and wait until it is registered in naming service
    #

    #print "ARGS = ",args
    if ('study' not in args['embedded']) | (args["gui"] == 0):
        print "RunStudy"
        myServer=SalomeDSServer(args)
        myServer.run()
        if sys.platform == "win32":
          clt.waitNS("/myStudyManager")
        else:
          clt.waitNSPID("/myStudyManager",myServer.PID)

    #
    # Launch LauncherServer
    #
    
    myCmServer = LauncherServer(args)
    myCmServer.setpath(modules_list,modules_root_dir)
    myCmServer.run()

    #
    # Launch ConnectionManagerServer
    #

    myConnectionServer = ConnectionManagerServer(args)
    myConnectionServer.run()


    from Utils_Identity import getShortHostName
    
    if os.getenv("HOSTNAME") == None:
        if os.getenv("HOST") == None:
            os.environ["HOSTNAME"]=getShortHostName()
        else:
            os.environ["HOSTNAME"]=os.getenv("HOST")

    theComputer = getShortHostName()
    
    #
    # Launch local C++ Container (FactoryServer),
    # and wait until it is registered in naming service
    #

    if ('cppContainer' in args['standalone']) | (args["gui"] == 0) : 
        myServer=ContainerCPPServer(args)
        myServer.run()
        if sys.platform == "win32":
          clt.waitNS("/Containers/" + theComputer + "/FactoryServer")
        else:
          clt.waitNSPID("/Containers/" + theComputer + "/FactoryServer",myServer.PID)

    #
    # Launch local Python Container (FactoryServerPy),
    # and wait until it is registered in naming service
    #

    if 'pyContainer' in args['standalone']:
        myServer=ContainerPYServer(args)
        myServer.run()
        if sys.platform == "win32":
          clt.waitNS("/Containers/" + theComputer + "/FactoryServerPy")
        else:
          clt.waitNSPID("/Containers/" + theComputer + "/FactoryServerPy",myServer.PID)

    #
    # Wait until Session Server is registered in naming service
    #
    
    if args["gui"]:
##----------------        
        import Engines
        import SALOME
        import SALOMEDS
        import SALOME_ModuleCatalog
        import SALOME_Session_idl
        if sys.platform == "win32":
          session=clt.waitNS("/Kernel/Session",SALOME.Session)
        else:
          session=clt.waitNSPID("/Kernel/Session",mySessionServ.PID,SALOME.Session)
        args["session_object"] = session
    end_time = os.times()
    if verbose(): print
    print "Start SALOME, elapsed time : %5.1f seconds"% (end_time[4]
                                                         - init_time[4])

    # ASV start GUI without Loader
    #if args['gui']:
    #    session.GetInterface()

    #
    # additionnal external python interpreters
    #
    nbaddi=0
    
    try:
        if 'interp' in args:
            nbaddi = args['interp']
    except:
        import traceback
        traceback.print_exc()
        print "-------------------------------------------------------------"
        print "-- to get an external python interpreter:runSalome --interp=1"
        print "-------------------------------------------------------------"
        
    if verbose(): print "additional external python interpreters: ", nbaddi
    if nbaddi:
        for i in range(nbaddi):
            print "i=",i
            anInterp=InterpServer(args)
            anInterp.run()

    # set PYTHONINSPECT variable (python interpreter in interactive mode)
    if args['pinter']:
        os.environ["PYTHONINSPECT"]="1"
        try:
            import readline
        except ImportError:
            pass
        
    return clt

# -----------------------------------------------------------------------------

def useSalome(args, modules_list, modules_root_dir):
    """
    Launch all SALOME servers requested by args,
    save list of process, give info to user,
    show registered objects in Naming Service.
    """
    global process_id
    
    clt=None
    try:
        clt = startSalome(args, modules_list, modules_root_dir)
    except:
        import traceback
        traceback.print_exc()
        print
        print
        print "--- Error during Salome launch ---"
        
    #print process_id

    from addToKillList import addToKillList
    from killSalomeWithPort import getPiDict

    filedict = getPiDict(args['port'])
    for pid, cmd in process_id.items():
        addToKillList(pid, cmd, args['port'])
        pass

    if verbose(): print """
    Saving of the dictionary of Salome processes in %s
    To kill SALOME processes from a console (kill all sessions from all ports):
      python killSalome.py 
    To kill SALOME from the present interpreter, if it is not closed :
      killLocalPort()      --> kill this session
                               (use CORBA port from args of runSalome)
      givenPortKill(port)  --> kill a specific session with given CORBA port 
      killAllPorts()       --> kill all sessions
    
    runSalome, with --killall option, starts with killing
    the processes resulting from the previous execution.
    """%filedict
    
    #
    #  Print Naming Service directory list
    #
    
    if clt != None:
        if verbose():
            print
            print " --- registered objects tree in Naming Service ---"
            clt.showNS()
            pass
        
        if not args['gui'] or not args['session_gui']:
            if args['shutdown_servers']:
                class __utils__(object):
                    def __init__(self, port):
                        self.port = port
                        import killSalomeWithPort
                        self.killSalomeWithPort = killSalomeWithPort
                        return
                    def __del__(self):
                        self.killSalomeWithPort.killMyPort(self.port)
                        return
                    pass
                args['shutdown_servers'] = __utils__(args['port'])
                pass
            pass
        
        # run python scripts, passed via --execute option
        toimport = []
        if args.has_key('pyscript'):
            if args.has_key('gui') and args.has_key('session_gui'):
                if not args['gui'] or not args['session_gui']:
                    toimport = args['pyscript']

        for srcname in toimport :
            if srcname == 'killall':
                clt.showNS()
                killAllPorts()
                sys.exit(0)
            else:
                if os.path.isabs(srcname):
                    if os.path.exists(srcname):
                        execScript(srcname)
                    elif os.path.exists(srcname+".py"):
                        execScript(srcname+".py")
                    else:
                        print "Can't execute file %s" % srcname
                    pass
                else:
                    found = False
                    for path in [os.getcwd()] + sys.path:
                        if os.path.exists(os.path.join(path,srcname)):
                            execScript(os.path.join(path,srcname))
                            found = True
                            break
                        elif os.path.exists(os.path.join(path,srcname+".py")):
                            execScript(os.path.join(path,srcname+".py"))
                            found = True
                            break
                        pass
                    if not found:
                        print "Can't execute file %s" % srcname
                        pass
                    pass
                pass
            pass
        pass
    return clt
    
def execScript(script_path):
    print 'executing', script_path
    sys.path.insert(0, os.path.dirname(script_path))
    execfile(script_path,globals())
    del sys.path[0]

# -----------------------------------------------------------------------------

def registerEnv(args, modules_list, modules_root_dir):
    """
    Register args, modules_list, modules_root_dir in a file
    for further use, when SALOME is launched embedded in an other application.
    """
    if sys.platform == "win32":
      fileEnv = os.getenv('TEMP')
    else:
      fileEnv = '/tmp/'

    fileEnv += os.getenv('USER') + "_" + str(args['port']) \
            + '_' + args['appname'].upper() + '_env'
    fenv=open(fileEnv,'w')
    pickle.dump((args, modules_list, modules_root_dir),fenv)
    fenv.close()
    os.environ["SALOME_LAUNCH_CONFIG"] = fileEnv

# -----------------------------------------------------------------------------

def searchFreePort(args, save_config=1):
    print "Searching for a free port for naming service:",
    #
    if sys.platform == "win32":
        tmp_file = os.getenv('TEMP');
    else:
        tmp_file = '/tmp'
    tmp_file = os.path.join(tmp_file, '.netstat_%s'%os.getpid())
    #
    ###status = os.system("netstat -ltn | grep -E :%s > /dev/null 2>&1"%(NSPORT))
    os.system( "netstat -a -n > %s" % tmp_file );
    f = open( tmp_file, 'r' );
    ports = f.readlines();
    f.close();
    os.remove( tmp_file );
    #
    def portIsUsed(port, data):
        regObj = re.compile( ".*tcp.*:([0-9]+).*:.*listen", re.IGNORECASE );
        for item in data:
            try:
                p = int(regObj.match(item).group(1))
                if p == port: return True
                pass
            except:
                pass
            pass
        return False
    #
    NSPORT=2810
    limit=NSPORT+100
    #
    while 1:
        if not portIsUsed(NSPORT, ports):
            print "%s - OK"%(NSPORT)
            #
            from salome_utils import generateFileName, getHostName
            hostname = getHostName()
            #
            home  = os.getenv("HOME")
            appli = os.getenv("APPLI")
            kwargs={}
            if appli is not None: 
              home = os.path.join(home, appli,"USERS")
              kwargs["with_username"]=True
            #
            omniorb_config = generateFileName(home, prefix="omniORB",
                                              extension="cfg",
                                              hidden=True,
                                              with_hostname=True,
                                              with_port=NSPORT,
                                              **kwargs)
            orbdata = []
            initref = "NameService=corbaname::%s:%s"%(hostname, NSPORT)
            from omniORB import CORBA
            if CORBA.ORB_ID == "omniORB4":
                orbdata.append("InitRef = %s"%(initref))
                orbdata.append("giopMaxMsgSize = 2097152000  # 2 GBytes")
                orbdata.append("traceLevel = 0 # critical errors only")
            else:
                orbdata.append("ORBInitRef %s"%(initref))
                orbdata.append("ORBgiopMaxMsgSize = 2097152000  # 2 GBytes")
                orbdata.append("ORBtraceLevel = 0 # critical errors only")
                pass
            orbdata.append("")
            f = open(omniorb_config, "w")
            f.write("\n".join(orbdata))
            f.close()
            #
            os.environ['OMNIORB_CONFIG'] = omniorb_config
            os.environ['NSPORT'] = "%s"%(NSPORT)
            os.environ['NSHOST'] = "%s"%(hostname)
            args['port'] = os.environ['NSPORT']
            #
            if save_config:
                last_running_config = generateFileName(home, prefix="omniORB",
                                                       suffix="last",
                                                       extension="cfg",
                                                       hidden=True,
                                                       **kwargs)
                try:
                    if sys.platform == "win32":
                        import shutil       
                        shutil.copyfile(omniorb_config, last_running_config)
                    else:
                        try:
                            os.remove(last_running_config)
                        except OSError:
                            pass
                        os.symlink(omniorb_config, last_running_config)
                        pass
                    pass
                except:
                    pass
            break
        print "%s"%(NSPORT),
        if NSPORT == limit:
            msg  = "\n"
            msg += "Can't find a free port to launch omniNames\n"
            msg += "Try to kill the running servers and then launch SALOME again.\n"
            raise RuntimeError, msg
        NSPORT=NSPORT+1
        pass
    return
    
# -----------------------------------------------------------------------------

def no_main():
    """Salome Launch, when embedded in other application"""
    fileEnv = os.environ["SALOME_LAUNCH_CONFIG"]
    fenv=open(fileEnv,'r')
    args, modules_list, modules_root_dir = pickle.load(fenv)
    fenv.close()
    kill_salome(args)
    searchFreePort(args, 0)
    clt = useSalome(args, modules_list, modules_root_dir)
    return clt

# -----------------------------------------------------------------------------

def main():
    """Salome launch as a main application"""
    from salome_utils import getHostName
    print "runSalome running on %s" % getHostName()
    args, modules_list, modules_root_dir = setenv.get_config()
    kill_salome(args)
    save_config = True
    if args.has_key('save_config'):
        save_config = args['save_config']
    # --
    test = True
    if args['wake_up_session']:
        test = False
        pass
    if test:
        searchFreePort(args, save_config)
        pass
    # --
    #setenv.main()
    setenv.set_env(args, modules_list, modules_root_dir)
    clt = useSalome(args, modules_list, modules_root_dir)
    return clt,args

# -----------------------------------------------------------------------------

def foreGround(clt, args):
    # --
    if "session_object" not in args:
        return
    session = args["session_object"]
    # --
    # Wait until gui is arrived
    # tmax = nbtot * dt
    # --
    gui_detected = False
    dt = 0.1
    nbtot = 100
    nb = 0
    while 1:
        try:
            status = session.GetStatSession()
            gui_detected = status.activeGUI
        except:
            pass
        if gui_detected:
            break
        from time import sleep
        sleep(dt)
        nb += 1
        if nb == nbtot:
            break
        pass
    # --
    if not gui_detected:
        return
    # --
    from salome_utils import getPortNumber
    port = getPortNumber()
    # --
    server = Server({})
    if sys.platform == "win32":
      server.CMD = [os.getenv("PYTHONBIN"), "-m", "killSalomeWithPort", "--spy", "%s"%(os.getpid()), "%s"%(port)]
    else:
      server.CMD = ["killSalomeWithPort.py", "--spy", "%s"%(os.getpid()), "%s"%(port)]
    server.run()   
    # os.system("killSalomeWithPort.py --spy %s %s &"%(os.getpid(), port))
    # --
    dt = 1.0
    try:
        while 1:
            try:
                status = session.GetStatSession()
                assert status.activeGUI
            except:
                break
            from time import sleep
            sleep(dt)
            pass
        pass
    except KeyboardInterrupt:
        from killSalomeWithPort import killMyPort
        killMyPort(port)
        pass
    return

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    import user
    clt,args = main()
    # --
    test = args['gui'] and args['session_gui']
    test = test or args['wake_up_session']
    # --
    # The next test covers the --pinter option or var PYTHONINSPECT setted
    # --
    test = test and not os.environ.get('PYTHONINSPECT')
    # --
    # The next test covers the python -i $KERNEL_ROOT_DIR/bin/salome/runSalome.py case
    # --
    try:
        from ctypes import POINTER, c_int, cast, pythonapi
        iflag_ptr = cast(pythonapi.Py_InteractiveFlag, POINTER(c_int))
        test = test and not iflag_ptr.contents.value
    except:
        pass
    # --
    test = test and os.getenv("SALOME_TEST_MODE", "0") != "1"
    test = test and args['foreground']
    # --
    if test:
        foreGround(clt, args)
        pass
    # --
    pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
"""
This module provides several functions to indicate the deprecation of a
module, a method or a function.

Example::

    from salome.kernel.deprecation import deprecated

    @deprecated("Deprecated since version 6.3.0. Consider replacement with "
                "newFunction()")
    def oldFunction():
      ...

"""

import sys
import warnings
import inspect
import os

from salome.kernel import termcolor

msg_seedoc = "See documentation for possible replacements."

def __deprecated_with_msg(func, msg):

    def new_func(*args, **kwargs):
        if len(inspect.stack()) > 1:
            callingfunc = inspect.stack()[1][3]
        else:
            callingfunc = "CORBA middleware"
        warnings.warn(
            ("Call to deprecated function %(funcname)s of module " +
             "%(modname)s (called from %(callingfunc)s).\n  %(msg)s") % {
                'funcname': func.__name__,
                'modname': func.__module__,
                'callingfunc': callingfunc,
                'msg': msg,
            },
            category = DeprecationWarning,
            stacklevel = 2
        )
        return func(*args, **kwargs)
    return new_func

def deprecated(msg = msg_seedoc):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used. The message in parameter will
    be displayed and should indicate how this function can be
    replaced. If the terminal can display colors, the warning
    messages will appear in blue.
    """
    def make_dep(f):
        if is_called_by_sphinx():
            return f
        else:
            g = __deprecated_with_msg(f, msg)
            g.__name__ = f.__name__
            g.__doc__ = f.__doc__
            g.__dict__.update(f.__dict__)
            return g
    return make_dep

def deprecated_module(msg = msg_seedoc):
    """
    This function can be used to mark a module as deprecated.
    It must be called explicitly at the beginning of the deprecated
    module. It will result in a warning being emitted. The message
    in parameter will be displayed and should indicate how this
    module can be replaced. If the terminal can display colors,
    the warning messages will appear in blue.
    """
    if not is_called_by_sphinx():
        warnings.warn(
            "Importation of deprecated module %(modname)s.\n  %(msg)s" % {
                'modname': inspect.getmodulename(inspect.stack()[1][1]),
                'msg': msg,
            },
            category = DeprecationWarning,
            stacklevel = 5
        )

def is_called_by_sphinx():
    """
    Determine if the calling code is ultimately called by sphinx to generate
    documentation. The result can be used to conditionally inhibit the
    decorators or some Salome-related imports that fail when called outside
    Salome.
    """
    calling_file = inspect.stack()[len(inspect.stack())-1][1]
    basename = os.path.basename(calling_file)
    return (basename == "sphinx-build")


def __show_colored_warning(message, category, filename,
                           lineno, file = sys.stderr, line = None):
    str = warnings.formatwarning(message, category, filename, lineno, line)
    if category == DeprecationWarning and termcolor.canDisplayColor(file):
        file.write(termcolor.makeColoredMessage(str, termcolor.BLUE))
    else:
        file.write(str)

# Enable warnings for deprecated functions and modules (in Python 2.7, they
# are disabled by default)
warnings.filterwarnings("always", "Call to *", DeprecationWarning)
warnings.filterwarnings("always", "Importation of *", DeprecationWarning)
warnings.showwarning = __show_colored_warning
